#ifndef VerilogDocGen_h
#define VerilogDocGen_h

#include "entry.h"
#include "verilogscanner.h"
#include "vhdlscanner.h"

class MyParserConv  
{
  
 public:
  uint iFileSize; 

  ~MyParserConv(){}
  MyParserConv(){}
  
 int parse(MyParserConv*);
 int doLex();
 
 
};



class VerilogDocGen  
{
public:

  enum VerilogClasses {ENTITYCLASS,PACKBODYCLASS,ARCHITECTURECLASS,PACKAGECLASS};
 	
  enum States {STATE_FUNCTION=0x100,STATE_MODULE,STATE_UDP,STATE_TASK,STATE_GENERATE};
  enum Ports {INPUT_PORT=0x200,OUTPUT_PORT,INOUT_PORT,GATE,NET};
 // enum Regs {INTEGER_REG=0x400,REG_REG,OUTPUT_REG,INOUT_REG,INPUT_REG};

	enum VerilogKeyWords
	{
	  MODULE=0x1000,
	  FUNCTION,
	  FEATURE,
	  INIT,
	  WIRE,
	  INTEGER,
	  REGISTER,
	  PRIMITIVE,
	  COMPONENT, //0x7
	  PORT,
      PARAMETER,
	  ALWAYS,
	  TRI,
	  NETTYPE,
	  INPUT,
	  TIME,
	  REALTIME,
	  REAL,
	  TASK,
	  OUTPUT,
	  DEFPARAM,
	  SPECPARAM,
	  GENERATE,
	  INOUT,
	  REPEAT,
	  INCLUDE
	  };

// functions for  verilog parser ---------------------

static void parseFunction(Entry* e,const char* s=NULL);
static void parseModule(const char* s=NULL);
static void parseReg(Entry* e,const char* s=NULL);
static void parseInteger(Entry* e,const char* s=NULL);
static void parsePortDir(Entry* e,int type,const char* s=NULL);
static void parseParam(Entry* e,const char* s=NULL);
static void parseModulInstance(Entry* e,const char* s=NULL);
static void parseListOfPorts(const char* s=NULL);
static void parseAlways(bool b=false);
static void parseModuleInst(QCString& first,QCString& sec);

static void deleteSpecChars(char* str,char *buf);
static void addSubEntry(Entry* root, Entry* e);

static QCString convertTypeToString(int type);

static void writeVHDLDeclarations(MemberList* ml,OutputList &ol,
               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
               const char *title,const char *subtitle,bool showEnumValues,int type);

static void writeVhdlDeclarations(MemberList* ml,OutputList& ol,GroupDef* gd,ClassDef* cd);

static void writePlainVHDLDeclarations(MemberDef* mdef,MemberList* mlist,OutputList &ol,
               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,int specifier);

static void writeVHDLDeclaration(MemberDef* mdef,OutputList &ol,
                   ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
                   bool inGroup);


static Entry* makeNewEntry(char* name=NULL,int sec=0,int spec=0,int line=0);

static MemberDef* findMember(QCString& className, QCString& memName,int type);

static MemberDef* findMemberDef(ClassDef* cd,QCString& key,MemberList::ListType type,int t,bool def=false);

static void setCurrVerilogClass(QCString&);

static MemberDef* findDefinition(ClassDef* cd, QCString& memName);
static QCString getClassTitle(const ClassDef*);
static void setBrief(QCString&,int);
static  Entry* getCurrBrief();
static int findKeyWord(const char*);
static void parseIdentVerilog();
static void addIncludes();
static void VerilogDocGen::initVerilogParser();

};


#endif