/******************************************************************************
 * Simple Parser for Verilog 2001 subset
 * written by M. Kreis
 * Date: 04/2008
 * supports the IEEE Std 1364-2001 (Revision of IEEE Std 1364-1995)Verilog subset
 *
 *****************************************************************************/

%locations
%skeleton "glr.c" 
%name-prefix="c_"
%debug

%{

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "VerilogDocGen.h"
#include "membergroup.h"
#include "VerilogParser.hpp"
#include "vhdldocgen.h"
#include "doxygen.h"
#include "searchindex.h"
#define YYDEBUG 1
#define YYMAXDEPTH 15000

MyParserConv* myconv=0;

static int CurrState=0;
static int          currVerilogType;

static Entry*		current_rootVerilog  ;
static Entry*		currentVerilog=0  ;
static Entry*       currentFunctionVerilog=0;
static Entry        prevDocEntryVerilog;

static bool         parseCode=FALSE;
static bool         isVerilogInst=false;
static bool         isPrevDocVerilog=false;

static QCString     currVerilogClass;
static QCString     identVerilog;
static QCString     currVerilogInst;

int c_lex (void);
void c_error (char const *);
void parseString();
void writeDigit();
void initVerilogParser();

bool findExtendsComponent(QList<BaseInfo> *extend,QCString& compName);

%}

%union {
	int itype;	/* for count */
	char ctype;	/* for char */
	char cstr[1024];
	}

%token NET_TOKEN STR0_TOKEN STR1_TOKEN GATE_TOKEN  STRING_TOKEN 
%token DIGIT_TOKEN UNDERSCORE_TOKEN SEM_TOKEN DOT_TOKEN 
%token LETTER_TOKEN  PLUS_TOKEN MINUS_TOKEN
%token COLON_TOKEN RBRACE_TOKEN LBRACE_TOKEN RBRACKET_TOKEN LBRACKET_TOKEN AND_TOKEN
%token OR_TOKEN EQU_TOKEN GT_TOKEN LT_TOKEN NOT_TOKEN  MULT_TOKEN PERCENTAL_TOKEN
%token ENV_TOKEN PARA_TOKEN CHAR_TOKEN AT_TOKEN DOLLAR_TOKEN BASE_TOKEN
%token SNAKE_TOKEN EXCLAMATION_TOKEN LRAM_TOKEN RRAM_TOKEN
%token PARAMETER_TOKEN  OUTPUT_TOKEN INOUT_TOKEN
%token SMALL_TOKEN MEDIUM_TOKEN LARGE_TOKEN VEC_TOKEN SCALAR_TOKEN REG_TOKEN TIME_TOKEN REAL_TOKEN  EVENT_TOKEN
%token ASSIGN_TOKEN DEFPARAM_TOKEN MODUL_TOKEN ENDMODUL_TOKEN MACRO_MODUL_TOKEN
%token ENDPRIMITIVE_TOKEN PRIMITIVE_TOKEN INITIAL_TOKEN TABLE_TOKEN ENDTABLE_TOKEN ALWAYS_TOKEN
%token TASK_TOKEN ENDTASK_TOKEN FUNC_TOKEN ENDFUNC_TOKEN
%token IF_TOKEN CASE_TOKEN CASEX_TOKEN CASEZ_TOKEN FOREVER_TOKEN REPEAT_TOKEN
%token FOR_TOKEN JOIN_TOKEN WAIT_TOKEN FORCE_TOKEN RELEASE_TOKEN DEASSIGN_TOKEN DISABLE_TOKEN
%token  WHILE_TOKEN ELSE_TOKEN ENDCASE_TOKEN BEGIN_TOKEN DEFAULT_TOKEN FORK_TOKEN
%token END_TOKEN SPECIFY_TOKEN ENDSPECIFY_TOKEN SPECPARAM_TOKEN
%token DSETUP_TOKEN DHOLD_TOKEN  DWIDTH_TOKEN DPERIOD_TOKEN DSKEW_TOKEN DRECOVERY_TOKEN DSETUPHOLD_TOKEN
%token POSEDGE_TOKEN NEGEDGE_TOKEN EDGE_TOKEN
%token COMMA_TOKEN  QUESTION_TOKEN AUTO_TOKEN
%token  INPUT_TOKEN SIGNED_TOKEN LOCALPARAM_TOKEN 
%token  INTEGER_TOKEN BEGIN_SPEC_TOKEN  NOCHANGE_TOKEN
%token  GENERATE_TOKEN ENDGENERATE_TOKEN DEFAULT_TOKEN GENVAR_TOKEN
%token  LIBRARY_TOKEN CONFIG_TOKEN ENDCONFIG_TOKEN INCLUDE_TOKEN PULSEON_DETECT_TOKEN PULSEONE_EVENT_TOKEN
%token  USE_TOKEN LIBLIST_TOKEN INSTANCE_TOKEN CELL_TOKEN SHOWCANCEL_TOKEN NOSHOWCANCEL_TOKEN
%token  REMOVAL_TOKEN FULLSKEW_TOKEN TIMESKEW_TOKEN RECREM_TOKEN
%token IFNONE_TOKEN REALTIME_TOKEN DESIGN_TOKEN FORK_SPEC_TOKEN

%start file 
/* -------------- rules section -------------- */
%%
file	: lines 	
	    ;
lines 	:{identVerilog.resize(0);} description  
           | lines  description {identVerilog.resize(0);}
		   | module_or_generate_item // for parsing inline functions/always/tasks
		   ;


//------------------------------------------------------------------------------------------------------
//----------------------------   A.1.1 Library source text  -------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

library_text : library_descriptions
              ; 
library_descriptions : library_declaration
                     | include_statement
                     | config_declaration
					 ;

library_declaration : LIBRARY_TOKEN SEM_TOKEN
                      

file_path_spec : simplexx
                | STRING_TOKEN
				;

include_statement :  INCLUDE_TOKEN GT_TOKEN file_path_spec LT_TOKEN SEM_TOKEN
                   | INCLUDE_TOKEN error SEM_TOKEN
				   ;
//------------------------------------------------------------------------------------------------------
//----------------------------  A.1.2 Configuration source text  -------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

config_declaration : CONFIG_TOKEN simplexx SEM_TOKEN  design_statement config_rule_statement_list ENDCONFIG_TOKEN
                   | CONFIG_TOKEN simplexx SEM_TOKEN  design_statement  ENDCONFIG_TOKEN      
				  
				  ; 

design_statement : DESIGN_TOKEN  aidentifier_list  SEM_TOKEN
                 | DESIGN_TOKEN error SEM_TOKEN
				 ;



config_rule_statement_list:config_rule_statement
                          | config_rule_statement_list config_rule_statement
						  ;


config_rule_statement : DEFAULT_TOKEN liblist_clause SEM_TOKEN
                      | inst_clause liblist_clause SEM_TOKEN
                      | inst_clause use_clause SEM_TOKEN
                      | cell_clause liblist_clause SEM_TOKEN
                      | cell_clause use_clause SEM_TOKEN
					  ;


aidentifier_list : simplexx
                | aidentifier_list simplexx
                ;


inst_clause :INSTANCE_TOKEN simplexx
            ;

cell_clause : CELL_TOKEN simplexx
            ;
             

liblist_clause : LIBLIST_TOKEN  
               | LIBLIST_TOKEN aidentifier_list 
               ;

use_clause : USE_TOKEN simplexx config 
		    ;  

config : /* empty */
       | COLON_TOKEN CONFIG_TOKEN


//------------------------------------------------------------------------------------------------------
//----------------------------  A.1.3 Module and primitive source text  -------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

description : module_declaration {}
	         | udp_declaration
             | library_text
			 ;


module_declaration : attribute_instance module_keyword    SEM_TOKEN module_option  end_mod
					| attribute_instance module_keyword   SEM_TOKEN  end_mod
					|  attribute_instance  module_keyword    module_parameter_port_list 
                       SEM_TOKEN  module_option  end_mod				
					|  attribute_instance module_keyword   list_of_ports  SEM_TOKEN  module_option  end_mod
                    |  attribute_instance module_keyword   list_of_port_declarations  SEM_TOKEN  module_option  end_mod
                    |  attribute_instance module_keyword   list_of_port_declarations  SEM_TOKEN  error end_mod
                 	|  attribute_instance module_keyword   list_of_ports  SEM_TOKEN  error  end_mod    
				  	|  attribute_instance module_keyword RBRACE_TOKEN LBRACE_TOKEN SEM_TOKEN module_option  end_mod
		      		|  attribute_instance module_keyword module_parameter_port_list   list_of_ports  SEM_TOKEN  module_option  end_mod
                    |  attribute_instance module_keyword  module_parameter_port_list list_of_port_declarations  SEM_TOKEN  module_option  end_mod
  
					;

					

name_of_module  : identifier 
                ;

module_keyword : MODUL_TOKEN name_of_module {
                                    if(!parseCode) { 
							              
											 currentVerilog=VerilogDocGen::makeNewEntry("",Entry::CLASS_SEC,VerilogDocGen::MODULE);
                                             currentVerilog->protection=Public;
					                         VerilogDocGen::parseModule();
							                 CurrState=VerilogDocGen::STATE_MODULE;
				                             VerilogDocGen::addSubEntry(current_rootVerilog,currentVerilog);
										    }
                                            else {
											      VerilogDocGen::parseModule();
                                         		  }
                               currVerilogType=0;						       
							   verilogBufferClear();
							 }
               | MACRO_MODUL_TOKEN 
			   ;

end_mod :  {
            if(!parseCode){ 
			        int ll=getVerilogLine();
	                currentVerilog->endBodyLine=ll;
		            mapVerilogFeatures(currentVerilog);
			       } 	 
              verilogBufferClear(); 
		   } ENDMODUL_TOKEN 

module_option : module_item
             | module_option module_item
		     ;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.1.4 Module parameters and ports  -------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

module_parameter_port_list : PARA_TOKEN RBRACE_TOKEN {currVerilogType=VerilogDocGen::PORT;} parameter_declaration_list  LBRACE_TOKEN               {currVerilogType=0;}
                           | PARA_TOKEN  error LBRACE_TOKEN 
						   ;

parameter_declaration_list:  PARAMETER_TOKEN  { currVerilogType=VerilogDocGen::PARAMETER;}   param_assignment                                      {currVerilogType=0;verilogBufferClear();}
                          | parameter_declaration_list COMMA_TOKEN PARAMETER_TOKEN  { currVerilogType=VerilogDocGen::PARAMETER;}  param_assignment {currVerilogType=0;verilogBufferClear();}
						  | parameter_declaration_list COMMA_TOKEN { currVerilogType=VerilogDocGen::PARAMETER;}param_assignment                    {currVerilogType=0;verilogBufferClear();}
						  ;



						  
list_of_ports :  RBRACE_TOKEN                    {currVerilogType=VerilogDocGen::PORT;} port_list LBRACE_TOKEN                                     {currVerilogType=0;}
               | RBRACE_TOKEN error LBRACE_TOKEN {currVerilogType=0;}  {verilogBufferClear();}
			  ;


list_of_port_declarations :  RBRACE_TOKEN task_port_list LBRACE_TOKEN {currVerilogType=0;}
                             ;


 port_list : port                       {VerilogDocGen::parseListOfPorts();verilogBufferClear();}
           | port_list COMMA_TOKEN port {VerilogDocGen::parseListOfPorts();verilogBufferClear();}
		   ;

port : port_expression 
     | DOT_TOKEN port_expression RBRACE_TOKEN LBRACE_TOKEN
	 | DOT_TOKEN port_expression RBRACE_TOKEN port_expression LBRACE_TOKEN 


port_expression : port_reference
                |  RRAM_TOKEN  port_reference_list LRAM_TOKEN
				;

port_reference_list: port_reference_list COMMA_TOKEN port_reference
                   | port_reference
                   ;

port_reference : identifier
               | DIGIT_TOKEN
			    | identifier RBRACKET_TOKEN expression LBRACKET_TOKEN
			   | identifier RBRACKET_TOKEN range_expression LBRACKET_TOKEN
			    ;

port_declaration : attribute_instance inout_declaration  { currVerilogType=0;}
                 | attribute_instance input_declaration  { currVerilogType=0;}
                 | attribute_instance output_declaration { currVerilogType=0;}
                 | error SEM_TOKEN	{ verilogBufferClear();}			
				;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.1.5 A.1.5 Module items  -------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

module_item : module_or_generate_item
            | port_declaration SEM_TOKEN
			|  port_declaration error SEM_TOKEN                {verilogBufferClear();}
            |  attribute_instance  generated_instantiation     {verilogBufferClear();}
            |  attribute_instance  local_parameter_declaration
            |  attribute_instance  parameter_declaration
            |  attribute_instance  specify_block               {verilogBufferClear();}
            |  attribute_instance  specparam_declaration



module_or_generate_item : module_or_generate_item_declaration 
                        |  attribute_instance  parameter_override  
                        |  attribute_instance  continuous_assign     {verilogBufferClear();}
                        |  attribute_instance  gate_instantiation    {verilogBufferClear();}
                        |  attribute_instance  udp_instantiation
                        |  attribute_instance  module_instantiation   
                        |  attribute_instance  initial_construct      {verilogBufferClear();}
                        |  attribute_instance  always_construct      
						;

module_or_generate_item_declaration : {if(parseCode) currVerilogType=VerilogDocGen::NETTYPE;} net_declaration {currVerilogType=0;}
                                    | reg_declaration
                                    | integer_declaration
                                    | real_declaration
                                    | time_declaration
                                    | realtime_declaration
                                    | event_declaration
                                    | genvar_declaration  {verilogBufferClear();}
                                    | task_declaration    {}
                                    | function_declaration{}
                                    ;


parameter_override : DEFPARAM_TOKEN { if(parseCode) currVerilogType=VerilogDocGen::DEFPARAM;} list_of_param_assignments SEM_TOKEN   {verilogBufferClear(); if(parseCode) currVerilogType=0; }
                   | DEFPARAM_TOKEN error SEM_TOKEN                                                                                 {verilogBufferClear(); if(parseCode) currVerilogType=0;} 
                   ;


//------------------------------------------------------------------------------------------------------
//---------task_port_type------------------- A.2.1 Declaration types
//-------------------------- A.2.1.1 Module parameter declarations  -------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

local_parameter_declaration : LOCALPARAM_TOKEN   range { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN             {currVerilogType=0;verilogBufferClear();}
                             | LOCALPARAM_TOKEN task_port_type { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN     {currVerilogType=0;verilogBufferClear();}
                             | LOCALPARAM_TOKEN   { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN                  {currVerilogType=0;verilogBufferClear();}
							 | LOCALPARAM_TOKEN signed  { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN            {currVerilogType=0;verilogBufferClear();}
							 | LOCALPARAM_TOKEN  signed range { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN      {currVerilogType=0;verilogBufferClear();}
							 | LOCALPARAM_TOKEN error SEM_TOKEN 
							 ;

parameter_declaration :       PARAMETER_TOKEN  { currVerilogType=VerilogDocGen::PARAMETER;}   list_of_param_assignments SEM_TOKEN                   {currVerilogType=0;verilogBufferClear();}
                             | PARAMETER_TOKEN  task_port_type { currVerilogType=VerilogDocGen::PARAMETER;}   list_of_param_assignments SEM_TOKEN   {currVerilogType=0;verilogBufferClear();}
                             |  PARAMETER_TOKEN  signed  range  { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN    {currVerilogType=0;verilogBufferClear();}
					         |  PARAMETER_TOKEN    range  { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN	        {currVerilogType=0;verilogBufferClear();}			
 					         |  PARAMETER_TOKEN    signed  { currVerilogType=VerilogDocGen::PARAMETER;} list_of_param_assignments SEM_TOKEN	        {currVerilogType=0;verilogBufferClear();}			
							 | PARAMETER_TOKEN error SEM_TOKEN {currVerilogType=0;verilogBufferClear();}
							 ;

specparam_declaration : SPECPARAM_TOKEN  range  list_of_specparam_assignments SEM_TOKEN  { verilogBufferClear();}
				      | SPECPARAM_TOKEN   list_of_specparam_assignments SEM_TOKEN        { verilogBufferClear();}
                      | SPECPARAM_TOKEN error SEM_TOKEN 
					  ;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.2.1.2 Port declarations ---------------------------------------------
//-----------------------------------------------------------------------------------------------------

inout_declaration :  INOUT_TOKEN   xsigned xrange          { currVerilogType=VerilogDocGen::INOUT; } identifier    {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | INOUT_TOKEN   net_type xsigned xrange { currVerilogType=VerilogDocGen::INOUT; } identifier    {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | inout_declaration COMMA_TOKEN xsigned xrange identifier                                       {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | inout_declaration COMMA_TOKEN net_type xsigned xrange identifier                              {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}			  
				   ;

input_declaration :  INPUT_TOKEN xsigned xrange          { currVerilogType=VerilogDocGen::INPUT; }  identifier      {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | INPUT_TOKEN net_type xsigned xrange { currVerilogType=VerilogDocGen::INPUT; }  identifier      {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | input_declaration COMMA_TOKEN xsigned xrange identifier                                        {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | input_declaration COMMA_TOKEN net_type xsigned xrange identifier                               {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
			      // | INPUT_TOKEN error SEM_TOKEN {verilogBufferClear();}
				   ;


output_declaration : OUTPUT_TOKEN  s_type { currVerilogType=VerilogDocGen::OUTPUT; }identifier                     {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | OUTPUT_TOKEN   { currVerilogType=VerilogDocGen::OUTPUT; }  identifier                         {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
				   | OUTPUT_TOKEN output_declaration net_assignment
				   | output_declaration COMMA_TOKEN  OUTPUT_TOKEN net_assignment
            	   | output_declaration COMMA_TOKEN identifier                                                     {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
			       | output_declaration COMMA_TOKEN OUTPUT_TOKEN s_type identifier                                 {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}		
			       | output_declaration COMMA_TOKEN OUTPUT_TOKEN  identifier                                       {if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}					   
			       ;

				  

output_var_type:TIME_TOKEN
               | INTEGER_TOKEN
			   ;


s_type : net_type
           | range
		   | signed
		   | REG_TOKEN
		   | s_type SIGNED_TOKEN
		   | s_type range
      	   | s_type output_var_type
		   ;



net_type : NET_TOKEN
		 ;

event_declaration : EVENT_TOKEN list_of_event_identifiers SEM_TOKEN{ verilogBufferClear();}
                   | EVENT_TOKEN error SEM_TOKEN { verilogBufferClear();} 
				   ;

genvar_declaration :GENVAR_TOKEN list_of_genvar_identifiers SEM_TOKEN
                   | GENVAR_TOKEN error SEM_TOKEN 
				    ;

integer_declaration :INTEGER_TOKEN { currVerilogType=VerilogDocGen::INTEGER; } list_of_variable_identifiers SEM_TOKEN {verilogBufferClear();}
                    | INTEGER_TOKEN error SEM_TOKEN 
					;


net_declaration : NET_TOKEN xscalared xsigned xrange    list_of_net_identifiers SEM_TOKEN                              {verilogBufferClear();} 
                | NET_TOKEN xscalared xsigned xrange     list_of_net_decl_assignments SEM_TOKEN                        {verilogBufferClear();} 
			    | NET_TOKEN xscalared xsigned xrange  delay3   list_of_net_decl_assignments SEM_TOKEN                  {verilogBufferClear();} 
			    | NET_TOKEN drive_strength xscalared xsigned xrange  delay3   list_of_net_decl_assignments SEM_TOKEN   {verilogBufferClear();} 
			     | NET_TOKEN drive_strength xscalared xsigned xrange    list_of_net_decl_assignments SEM_TOKEN          {verilogBufferClear();} 
			    | NET_TOKEN drive_strength xscalared xsigned xrange    list_of_net_identifiers SEM_TOKEN          {verilogBufferClear();}  
			    
				| NET_TOKEN charge_strength xscalared xsigned xrange     list_of_net_identifiers SEM_TOKEN        {verilogBufferClear();} 
			    | NET_TOKEN charge_strength xscalared xsigned xrange delay3    list_of_net_identifiers SEM_TOKEN        {verilogBufferClear();} 			  
			    | NET_TOKEN charge_strength xscalared xsigned xrange     list_of_net_decl_assignments SEM_TOKEN  {verilogBufferClear();} 
			  
			    | NET_TOKEN  error SEM_TOKEN                                                                     {verilogBufferClear();} 
			   	;


xscalared:/*empty*/ 
		| scalared
		;


scalared: VEC_TOKEN 
		| SCALAR_TOKEN
		;

real_declaration : REAL_TOKEN { currVerilogType=VerilogDocGen::INTEGER;} list_of_variable_identifiers SEM_TOKEN  {verilogBufferClear();currVerilogType=0;} 
                 | REAL_TOKEN error SEM_TOKEN                      {verilogBufferClear();} 
				 ;
realtime_declaration : REALTIME_TOKEN { currVerilogType=VerilogDocGen::TIME; }list_of_variable_identifiers SEM_TOKEN {verilogBufferClear();currVerilogType=0;} 
                     | REALTIME_TOKEN error SEM_TOKEN {verilogBufferClear();}
					 ; 

reg_declaration : REG_TOKEN  xsigned  range  {currVerilogType=VerilogDocGen::REGISTER; }list_of_variable_identifiers SEM_TOKEN {currVerilogType=0;}
                | REG_TOKEN   xsigned        {currVerilogType=VerilogDocGen::REGISTER;} list_of_variable_identifiers SEM_TOKEN {currVerilogType=0;}
                | REG_TOKEN error SEM_TOKEN { verilogBufferClear();}
				 ;

time_declaration : TIME_TOKEN { currVerilogType=VerilogDocGen::TIME; } list_of_variable_identifiers SEM_TOKEN  {verilogBufferClear();currVerilogType=0;} 
                 | TIME_TOKEN error SEM_TOKEN                          {verilogBufferClear();} 
				 ;
//------------------------------------------------------------------------------------------------------
//---------------------------- 2.2.1 Net and variable types--------------------------------------------
//-----------------------------------------------------------------------------------------------------

dimension_list: dimension
              | dimension_list dimension
			   ;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.2.2.2 Strengths --------------------------------------------
//-----------------------------------------------------------------------------------------------------

drive_strength:  RBRACE_TOKEN STR0_TOKEN COMMA_TOKEN STR1_TOKEN LBRACE_TOKEN
	            | RBRACE_TOKEN STR1_TOKEN COMMA_TOKEN STR0_TOKEN LBRACE_TOKEN
             //   | RBRACE_TOKEN error LBRACE_TOKEN
			//	| drive_strength SIGNED_TOKEN
				;


charge_strength : RBRACE_TOKEN SMALL_TOKEN LBRACE_TOKEN
             	| RBRACE_TOKEN MEDIUM_TOKEN LBRACE_TOKEN
	            | RBRACE_TOKEN LARGE_TOKEN LBRACE_TOKEN
           		;


//------------------------------------------------------------------------------------------------------
//----------------------------A.2.2.3 Delays --------------------------------------------
//-----------------------------------------------------------------------------------------------------


delay3 :  PARA_TOKEN RBRACE_TOKEN delay_value_list LBRACE_TOKEN {}
        | PARA_TOKEN delay_value 
	   ;

delay_value_list : delay_value 
                 | delay_value_list COMMA_TOKEN delay_value
				 ;

delay2 : delay3
       ;
	   
delay_value : mintypemax_expression
            ;

//------------------------------------------------------------------------------------------------------
//----------------------------A.2.3 Declaration lists --------------------------------------------
//-----------------------------------------------------------------------------------------------------

list_of_event_identifiers : list_of_event_lists  
						    | super_list
							;

list_of_event_lists:identifier
                   |list_of_event_lists COMMA_TOKEN identifier
                   ;


super_list: identifier dimension_list
          | super_list COMMA_TOKEN identifier dimension_list
		   ;
		    

list_of_genvar_identifiers : identifier 
                           | list_of_genvar_identifiers COMMA_TOKEN identifier
						    ;
							
list_of_net_decl_assignments :net_decl_assignment  
                              | list_of_net_decl_assignments COMMA_TOKEN net_decl_assignment  
 							  ;

list_of_net_identifiers : list_of_event_identifiers
                        ;

list_of_param_assignments : param_assignment 
                          | list_of_param_assignments COMMA_TOKEN param_assignment 
						  ;

						 

real_type : identifier 
          | identifier EQU_TOKEN expression 
          | identifier dimension_list
		  ;

variable_type:real_type
              ;


list_of_specparam_assignments : specparam_assignment 
							   | list_of_specparam_assignments COMMA_TOKEN specparam_assignment 				
                               ;

list_of_variable_identifiers  : variable_type                                          {if(!parseCode) {
                                                                                            const char*c1=$<cstr>1;
																						
																							VerilogDocGen::parseReg(currentVerilog);}
																							verilogBufferClear();} 
						      |	list_of_variable_identifiers COMMA_TOKEN variable_type {if(!parseCode) VerilogDocGen::parseReg(currentVerilog);verilogBufferClear();} 
							  ;
							  
  
//------------------------------------------------------------------------------------------------------
//----------------------------A.2.4 Declaration assignments --------------------------------------------
//-----------------------------------------------------------------------------------------------------
      
net_decl_assignment :identifier EQU_TOKEN expression
                    ; 
param_assignment :simplexx EQU_TOKEN expression {
                          			 if(currVerilogType==VerilogDocGen::PARAMETER && !parseCode)
									 VerilogDocGen::parseParam(currentVerilog);
									 verilogBufferClear();
	                   }
                 ;
				 
specparam_assignment : identifier EQU_TOKEN mintypemax_expression
                   //  | PATHPULSE_TOKEN EQU_TOKEN constant_mintypmax_expression 
					  ;

//------------------------------------------------------------------------------------------------------
//----------------------------A.2.5 Declaration ranges --------------------------------------------
//-----------------------------------------------------------------------------------------------------

dimension : RBRACKET_TOKEN dimension_constant_expression COLON_TOKEN dimension_constant_expression LBRACKET_TOKEN
           ;

range :  RBRACKET_TOKEN msb_constant_expression COLON_TOKEN lsb_constant_expression LBRACKET_TOKEN
       ;


//------------------------------------------------------------------------------------------------------
//----------------------------A.2.6 Function declarations --------------------------------------------
//-----------------------------------------------------------------------------------------------------

function_declaration : FUNC_TOKEN  automatic xsigned  range_or_type  name_of_function SEM_TOKEN 
                        function_item_declaration_list 
                        function_statement
                        ENDFUNC_TOKEN                  {if(!parseCode && currentFunctionVerilog)
						                                {
														  currentFunctionVerilog->endBodyLine=getVerilogPrevLine();
														} verilogBufferClear(); }

					  | FUNC_TOKEN  automatic xsigned range_or_type name_of_function RBRACE_TOKEN function_port_list LBRACE_TOKEN SEM_TOKEN 
                        function_item_declaration_list 
                        function_statement
                        ENDFUNC_TOKEN                  {if(!parseCode && currentFunctionVerilog){currentFunctionVerilog->endBodyLine=getVerilogPrevLine();} verilogBufferClear(); }
					  | FUNC_TOKEN error ENDFUNC_TOKEN { verilogBufferClear(); }
					  ;
 

name_of_function : identifier {  if(!parseCode){
                             //    printf("\n  funcname [%s] --\n",getVerilogString());
                                 currentFunctionVerilog=VerilogDocGen::makeNewEntry("",Entry::FUNCTION_SEC,VerilogDocGen::FUNCTION);
								 currentFunctionVerilog->fileName=getVerilogParsingFile();
								 VerilogDocGen::parseFunction(currentFunctionVerilog);
							     VerilogDocGen::addSubEntry(currentVerilog,currentFunctionVerilog);
								 CurrState=VerilogDocGen::STATE_FUNCTION;
								 }
								 verilogBufferClear();
							   }
        
				 ;


range_or_type : /*empty*/
              | range
	          | INTEGER_TOKEN {currVerilogType=VerilogDocGen::INTEGER;}
	          | REAL_TOKEN
			  | TIME_TOKEN
              | REALTIME_TOKEN
			  ;


 automatic: /*empty*/
          | AUTO_TOKEN
		  ;

xsigned: /*empty*/
       | SIGNED_TOKEN
	   ;

 function_item_declaration_list :  function_item_declaration
							     | function_item_declaration_list   function_item_declaration
								 ;
								 
			
function_item_declaration : block_item_declaration
                          | tf_input_declaration_list SEM_TOKEN { currVerilogType=0;}
                  		   ;


function_port_list : tf_input_declaration// attribute_instance tf_input_declaration 
                   | function_port_list COMMA_TOKEN attribute_instance tf_input_declaration
                    ;


//------------------------------------------------------------------------------------------------------
//---------------------------- A.2.7 Task declarations  --------------------------------------------
//-----------------------------------------------------------------------------------------------------

task_declaration : TASK_TOKEN   automatic  name_of_function SEM_TOKEN 
                    task_item_declaration_list statement_list ENDTASK_TOKEN                                      {if(!parseCode && currentFunctionVerilog) { currentFunctionVerilog->endBodyLine=getVerilogPrevLine();} verilogBufferClear();}
                | TASK_TOKEN   automatic  name_of_function SEM_TOKEN 
                    statement_list ENDTASK_TOKEN                 
				|  TASK_TOKEN  automatic  name_of_function  RBRACE_TOKEN  task_port_list LBRACE_TOKEN SEM_TOKEN 
                    block_item_declaration_list statement_list ENDTASK_TOKEN                                     {if(!parseCode && currentFunctionVerilog) { currentFunctionVerilog->endBodyLine=getVerilogPrevLine();} verilogBufferClear();}
                 |  TASK_TOKEN  automatic  name_of_function RBRACE_TOKEN  task_port_list LBRACE_TOKEN SEM_TOKEN 
                     statement_list ENDTASK_TOKEN                                                                {if(!parseCode && currentFunctionVerilog) { currentFunctionVerilog->endBodyLine=getVerilogPrevLine();} verilogBufferClear();}
				 | TASK_TOKEN error ENDTASK_TOKEN                                                                { verilogBufferClear();}
				 ;

block_item_declaration_list :block_item_declaration_list block_item_declaration
							|  block_item_declaration
							;

task_item_declaration_list :task_item_declaration_list task_item_declaration
							|  task_item_declaration
                             ;


task_item_declaration : block_item_declaration
                         | task_port_item SEM_TOKEN
						 ;


task_port_list :task_port_item 
               | task_port_list COMMA_TOKEN task_port_item 
			   ;
			   


task_port_item :  tf_port_declaration 
               ;


tf_port_declaration : tf_port_dir xsigned xreg  xrange  identifier {
                             if(!parseCode){
							   VerilogDocGen::parsePortDir(currentVerilog,3);}
							   verilogBufferClear();
                   }
                	 | tf_port_dir task_port_type  identifier  {if(!parseCode){
							   VerilogDocGen::parsePortDir(currentVerilog,3);}
							   verilogBufferClear();
                   }
                  | tf_port_declaration COMMA_TOKEN task_port_type  identifier  {if(!parseCode){
							   VerilogDocGen::parsePortDir(currentVerilog,3);}
							   verilogBufferClear();
                   }
				 
				 	 | tf_port_declaration COMMA_TOKEN xsigned xreg  xrange identifier {
                             if(!parseCode){
							   VerilogDocGen::parsePortDir(currentVerilog,3);}
							   verilogBufferClear();
                   }
					 | tf_port_declaration COMMA_TOKEN tf_port_dir xsigned xreg  xrange  identifier {
                             if(!parseCode){
							   VerilogDocGen::parsePortDir(currentVerilog,3);}
							   verilogBufferClear();
                   }
				//  |tf_port_dir error SEM_TOKEN
					 ; 

xreg:/*empty*/
    | REG_TOKEN

xrange:/*empty*/
      | range
	  ;
xnettype:/*empty*/
     | NET_TOKEN
     ;

tf_port_dir: attribute_instance INOUT_TOKEN   xnettype    { if(!parseCode )currVerilogType=VerilogDocGen::INOUT;}
           |  attribute_instance OUTPUT_TOKEN xnettype  { if(!parseCode)currVerilogType=VerilogDocGen::OUTPUT;}
		   |  attribute_instance INPUT_TOKEN  xnettype { if(!parseCode)currVerilogType=VerilogDocGen::INPUT;}
		   |  tf_port_dir COMMA_TOKEN identifier { if(!parseCode){VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}} 
		   ;


tf_input_declaration_list:tf_input_declaration
                         | tf_input_declaration_list COMMA_TOKEN tf_input_declaration
					 	 ;

tf_input_declaration : INPUT_TOKEN   xsigned   xrange  identifier                                         {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
                     | INPUT_TOKEN   REG_TOKEN  xsigned  xrange  identifier                               {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
                     | INPUT_TOKEN task_port_type  identifier                                             {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
                     | tf_input_declaration COMMA_TOKEN identifier                                        {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
				     | tf_input_declaration COMMA_TOKEN INPUT_TOKEN xsigned xrange identifier             {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
					 | tf_input_declaration COMMA_TOKEN INPUT_TOKEN REG_TOKEN xsigned xrange identifier   {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
				     | tf_input_declaration COMMA_TOKEN INPUT_TOKEN task_port_type identifier             {if(!parseCode){currVerilogType=VerilogDocGen::INPUT;VerilogDocGen::parsePortDir(currentVerilog,3);}verilogBufferClear();}
				    | INPUT_TOKEN error SEM_TOKEN { verilogBufferClear(); }  
					 ; 


task_port_type : TIME_TOKEN      
               | REAL_TOKEN     
			   | REALTIME_TOKEN  
			   | INTEGER_TOKEN   
			   ;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.2.8 Block item declarations   --------------------------------------------
//-----------------------------------------------------------------------------------------------------

block_item_declaration :  attribute_instance  block_reg_declaration
                       |  attribute_instance  event_declaration
                       |  attribute_instance  integer_declaration
                       |  attribute_instance  local_parameter_declaration 
                       |  attribute_instance  parameter_declaration
                       |  attribute_instance  real_declaration
                       |  attribute_instance  realtime_declaration
                       |  attribute_instance  time_declaration
                       ;

block_reg_declaration : REG_TOKEN    range list_of_block_variable_identifiers SEM_TOKEN
                      | REG_TOKEN   signed range list_of_block_variable_identifiers SEM_TOKEN
					  | REG_TOKEN    list_of_block_variable_identifiers SEM_TOKEN
                      ;


list_of_block_variable_identifiers : block_variable_type 
                                   | list_of_block_variable_identifiers COMMA_TOKEN block_variable_type 
                                   ;

block_variable_type : identifier
                    | identifier dimension_list
					;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.3.1 Primitive instantiation and instances    --------------------------------------------
//-----------------------------------------------------------------------------------------------------

gate_instantiation : GATE_TOKEN delay3 cmos_switch_instance_list SEM_TOKEN 
				   | GATE_TOKEN cmos_switch_instance_list SEM_TOKEN
				   | GATE_TOKEN drive_strength cmos_switch_instance_list SEM_TOKEN				  
				   | GATE_TOKEN drive_strength delay3 cmos_switch_instance_list SEM_TOKEN				  
				
				 //  | GATE_TOKEN  drive_strength delay3 enable_gate_instance_list SEM_TOKEN
				 //  | GATE_TOKEN  delay3 enable_gate_instance_list SEM_TOKEN
				 //  | GATE_TOKEN  drive_strength enable_gate_instance_list SEM_TOKEN
				 //  | GATE_TOKEN  enable_gate_instance_list SEM_TOKEN
			       | GATE_TOKEN pull_gate_instance_list SEM_TOKEN           
				   | GATE_TOKEN error SEM_TOKEN { verilogBufferClear(); }
				   ;

pull_gate_instance_list: pull_gate_instance
                       | pull_gate_instance_list COMMA_TOKEN pull_gate_instance
					   ;

cmos_switch_instance_list:cmos_switch_instance
                         | cmos_switch_instance_list COMMA_TOKEN cmos_switch_instance
                          ;



cmos_switch_instance :name_of_gate_instance  RBRACE_TOKEN output_terminal COMMA_TOKEN expression_list LBRACE_TOKEN
                     | RBRACE_TOKEN output_terminal COMMA_TOKEN expression_list LBRACE_TOKEN
                     ; 



pull_gate_instance : name_of_gate_instance  RBRACE_TOKEN output_terminal LBRACE_TOKEN
                   | RBRACE_TOKEN output_terminal LBRACE_TOKEN
				   ;  

name_of_gate_instance : identifier range 
                      | identifier
					  ;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.3.2 Primitive strengths  --------------------------------------------
//-----------------------------------------------------------------------------------------------------


output_terminal : net_lvalue
                ;

//------------------------------------------------------------------------------------------------------
//---------------------------- A.4.1 Module instantiation  --------------------------------------------
//-----------------------------------------------------------------------------------------------------

module_instantiation : module_ii11
					 ;

module_ii11          :  identifier { 
                                   /* 
									 currVerilogInst=QCString($<cstr>1); 
									 const char* ss=getVerilogString();
									 if(parseCode){
						 	           currVerilogType=VerilogDocGen::COMPONENT;
						 	          // verilogBufferClear();
							          }					
									*/
									} parameter_value_assignment  {verilogBufferClear();}
					 | identifier  module_instance_list {verilogBufferClear();} SEM_TOKEN 
					 ;

module_instance_list:  module_instance
                      | module_instance_list COMMA_TOKEN module_instance 
                     ;

parameter_value_assignment : PARA_TOKEN RBRACE_TOKEN list_of_parameter_assignments LBRACE_TOKEN  module_instance_list SEM_TOKEN
                            |  PARA_TOKEN error  module_instance_list SEM_TOKEN
                            ;
list_of_parameter_assignments : ordered_parameter_assignment_list
                               | named_parameter_assignment_list
                               ;

ordered_parameter_assignment_list:ordered_parameter_assignment
								 | ordered_parameter_assignment_list COMMA_TOKEN ordered_parameter_assignment 
                                 ;

ordered_parameter_assignment : expression
                             ;

 named_parameter_assignment_list :  named_parameter_assignment 
                                 |  named_parameter_assignment_list COMMA_TOKEN named_parameter_assignment 
                                 ;
                             

named_parameter_assignment : DOT_TOKEN identifier RBRACE_TOKEN  expression  LBRACE_TOKEN {}
                           | DOT_TOKEN identifier RBRACE_TOKEN    LBRACE_TOKEN {}
                           ;


module_instance : identifier11  RBRACE_TOKEN  list_of_port_connections  LBRACE_TOKEN {verilogBufferClear(); currVerilogType=0;} 
             	  | identifier11 RBRACE_TOKEN   LBRACE_TOKEN                        {verilogBufferClear(); currVerilogType=0;} 
				 ;

identifier11:identifier xrange { 
                            const char *name=$<cstr>1;
                            QCString first(name);
							QCString sec(getVerilogString());
						     QCString ll=getLastLetter();
							
							 VerilogDocGen::parseModuleInst(first,sec);
							     if(parseCode){
							  currVerilogType=VerilogDocGen::COMPONENT;
							  verilogBufferClear();
							  }
							  }
            // | identifier range;        
            ;

				 
list_of_port_connections : ordered_port_connection_list 
                         | named_port_connection_list 
						 ;
						 
ordered_port_connection_list : ordered_port_connection
                             | ordered_port_connection_list COMMA_TOKEN ordered_port_connection
							 ;

ordered_port_connection :   expression 
                       
						;
							 
named_port_connection_list : named_port_connection
                           | named_port_connection_list COMMA_TOKEN named_port_connection
						   ;

named_port_connection : attribute_instance named_parameter_assignment
                      ;

 

//------------------------------------------------------------------------------------------------------
//---------------------------- A.4.2 Generated instantiation  --------------------------------------------
//-----------------------------------------------------------------------------------------------------
//generated_instantiation ::= generate { generate_item } endgenerate

generated_instantiation :  GENERATE_TOKEN {CurrState=VerilogDocGen::STATE_GENERATE;}  generate_item_list  ENDGENERATE_TOKEN {CurrState=0;}
            					              |  GENERATE_TOKEN error  ENDGENERATE_TOKEN
            					              ;
            					              
generate_item_list :generate_item
                   | generate_item_list generate_item
				   ;
   
   
generate_item_or_null : {verilogBufferClear();} generate_item   
                      | SEM_TOKEN
			           ;		  


generate_item : generate_conditional_statement 
              | generate_case_statement        
              | generate_loop_statement        
              | generate_block                 
              | module_or_generate_item       
			  ;

generate_conditional_statement  :  IF_TOKEN RBRACE_TOKEN  expression LBRACE_TOKEN generate_item_or_null 
                                 |   IF_TOKEN RBRACE_TOKEN  expression LBRACE_TOKEN generate_item_or_null  ELSE_TOKEN  generate_item_or_null 
     						     ;

generate_case_statement :CASE_TOKEN  RBRACE_TOKEN  expression LBRACE_TOKEN  genvar_module_case_item_list  ENDCASE_TOKEN    
							   ;

genvar_module_case_item_list  :  genvar_case_item  
							  |  genvar_module_case_item_list COMMA_TOKEN genvar_case_item 
							  ;

genvar_case_item   :  expression_list COLON_TOKEN  generate_item_or_null
                   | DEFAULT_TOKEN   COLON_TOKEN  generate_item_or_null
                   | DEFAULT_TOKEN   generate_item_or_null
				   ;


generate_loop_statement  : FOR_TOKEN  RBRACE_TOKEN  genvar_assignment SEM_TOKEN  expression SEM_TOKEN  genvar_assignment  LBRACE_TOKEN generate_block
                         ;

genvar_assignment : identifier EQU_TOKEN expression
                  ;

generate_block  :  BEGIN_SPEC_TOKEN  generate_item_list  END_TOKEN  
                             |  BEGIN_TOKEN generate_item_list END_TOKEN
                              ;


//------------------------------------------------------------------------------------------------------
//---------------------------- A.5.1 UDP declaration  --------------------------------------------
//-----------------------------------------------------------------------------------------------------

udp_declaration : attribute_instance  PRIMITIVE_TOKEN name_of_udp  RBRACE_TOKEN { currVerilogType=VerilogDocGen::REGISTER} udp_port_list LBRACE_TOKEN SEM_TOKEN
                  udp_port_declaration_list udp_body ENDPRIMITIVE_TOKEN
                | attribute_instance  PRIMITIVE_TOKEN name_of_udp RBRACE_TOKEN udp_declaration_port_list LBRACE_TOKEN SEM_TOKEN
                  udp_port_declaration_list udp_body ENDPRIMITIVE_TOKEN
               // | PRIMITIVE_TOKEN error ENDPRIMITIVE_TOKEN {verilogBufferClear();}
				;

name_of_udp:        identifier { if(!parseCode) { 
							               //  printf("\n  name_of_mod [%s] [%d]--\n",getVerilogString(),getVerilogLine());
                                             currentVerilog=VerilogDocGen::makeNewEntry("",Entry::CLASS_SEC,VerilogDocGen::MODULE);
                                             currentVerilog->protection=Private;
					                          currentVerilog->stat=TRUE;
					                         VerilogDocGen::parseModule();
							                 CurrState=VerilogDocGen::STATE_MODULE;
				                             VerilogDocGen::addSubEntry(current_rootVerilog,currentVerilog);
										    }
                                            else {
											      VerilogDocGen::parseModule();
                                              //    currVerilogType=VerilogDocGen::MODULE;
												  }
						        verilogBufferClear();
							 }
                         ;


udp_port_list : identifier                            {VerilogDocGen::parseListOfPorts();verilogBufferClear();}
              | udp_port_list COMMA_TOKEN  identifier {VerilogDocGen::parseListOfPorts();verilogBufferClear();}
			  ;
udp_declaration_port_list : udp_output_declaration COMMA_TOKEN  udp_input_declaration_list
                          ;

udp_input_declaration_list:udp_input_declaration
                           | udp_input_declaration_list COMMA_TOKEN udp_input_declaration
                           | udp_input_declaration_list COMMA_TOKEN identifier 
							;

udp_port_declaration_list:udp_port_declaration
                         | udp_port_declaration_list  udp_port_declaration
						 ;  

udp_port_declaration : udp_output_declaration SEM_TOKEN
                     | udp_input_declaration SEM_TOKEN
                     | udp_reg_declaration SEM_TOKEN
                 	
					 | udp_port_declaration SEM_TOKEN
					  ;

udp_output_declaration : attribute_instance  OUTPUT_TOKEN identifier                                    { currVerilogType=VerilogDocGen::OUTPUT;if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
                       | attribute_instance  OUTPUT_TOKEN REG_TOKEN identifier                          { currVerilogType=VerilogDocGen::OUTPUT;if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
                       | attribute_instance  OUTPUT_TOKEN REG_TOKEN identifier  EQU_TOKEN expression    { currVerilogType=VerilogDocGen::OUTPUT;if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
                       | udp_output_declaration COMMA_TOKEN identifier                                  { currVerilogType=VerilogDocGen::OUTPUT;if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
					    ;

udp_input_declaration : attribute_instance  INPUT_TOKEN identifier        { currVerilogType=VerilogDocGen::INPUT;if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
                       | udp_input_declaration COMMA_TOKEN identifier     { currVerilogType=VerilogDocGen::INPUT;if(!parseCode)VerilogDocGen::parsePortDir(currentVerilog,3);verilogBufferClear();}
                       ;
udp_reg_declaration :  attribute_instance  REG_TOKEN identifier              {currVerilogType=VerilogDocGen::REGISTER;if(!parseCode) VerilogDocGen::parseReg(currentVerilog);verilogBufferClear();} 
                     | udp_reg_declaration COMMA_TOKEN identifier            {currVerilogType=VerilogDocGen::REGISTER;if(!parseCode) VerilogDocGen::parseReg(currentVerilog);verilogBufferClear();} 
					 ;
//------------------------------------------------------------------------------------------------------
//----------------------------5.1 body  --------------------------------------------
//-----------------------------------------------------------------------------------------------------

udp_body :  combinational_body 
          // | sequential_body
           ;

combinational_body : TABLE_TOKEN combinational_entry_list ENDTABLE_TOKEN                         { verilogBufferClear();} 
                    |  TABLE_TOKEN error ENDTABLE_TOKEN                                          { verilogBufferClear();} 
				    | udp_initial_statement  TABLE_TOKEN combinational_entry_list ENDTABLE_TOKEN { verilogBufferClear();}
					;

combinational_entry_list:combinational_entry
                        | combinational_entry_list combinational_entry
                         ;

combinational_entry : edge_input_list COLON_TOKEN  output_symbol SEM_TOKEN
                    | edge_input_list COLON_TOKEN current_state COLON_TOKEN next_state SEM_TOKEN
					;



udp_initial_statement : INITIAL_TOKEN identifier EQU_TOKEN init_val SEM_TOKEN { verilogBufferClear(); }
                      | INITIAL_TOKEN error SEM_TOKEN                         { verilogBufferClear(); }
					   ;

init_val : DIGIT_TOKEN { identVerilog+=$<cstr>1;writeDigit(); } 
         ;

				   
edge_input_list :edge_indicator
                | level_symbol
                | edge_input_list edge_indicator 
			    | edge_input_list level_symbol
				; 

edge_indicator : RBRACE_TOKEN level_symbol LBRACE_TOKEN
               | RBRACE_TOKEN level_symbol level_symbol LBRACE_TOKEN
			   |  edge_indicator RBRACE_TOKEN level_symbol LBRACE_TOKEN
			  // | error LBRACE_TOKEN
			   ;

current_state :level_symbol
              ; 
next_state : output_symbol 
           | MINUS_TOKEN
		    ;

output_symbol : level_symbol


level_symbol : DIGIT_TOKEN { identVerilog+=$<cstr>1;writeDigit(); } 
             | QUESTION_TOKEN
			 | MULT_TOKEN
			 | LETTER_TOKEN 

			 ;

//------------------------------------------------------------------------------------------------------
//----------------------------A.5.4 UDP instantiation  --------------------------------------------
//-----------------------------------------------------------------------------------------------------

udp_instantiation : identifier drive_strength  delay2  udp_instance_list SEM_TOKEN
                  | identifier drive_strength  udp_instance_list SEM_TOKEN
                  | identifier   udp_instance_list SEM_TOKEN
				  ;

udp_instance_list:udp_instance
                 | udp_instance_list COMMA_TOKEN udp_instance
				 ;

udp_instance : name_of_instance RBRACE_TOKEN output_terminal COMMA_TOKEN expression_list LBRACE_TOKEN
             | RBRACE_TOKEN output_terminal COMMA_TOKEN expression_list LBRACE_TOKEN
			 ;

name_of_instance :identifier  xrange 
                 //| identifier
				 ;

//------------------------------------------------------------------------------------------------------
//----------------------------A.6 Behavioral statements --------------------------------------------
//-----------------------------------------------------------------------------------------------------

continuous_assign : ASSIGN_TOKEN  drive_strength  delay3  list_of_net_assignments SEM_TOKEN
                  | ASSIGN_TOKEN    delay3  list_of_net_assignments SEM_TOKEN
				  | ASSIGN_TOKEN     list_of_net_assignments SEM_TOKEN
				  | ASSIGN_TOKEN  drive_strength  list_of_net_assignments SEM_TOKEN              
				  ;

list_of_net_assignments : net_assignment 
                        | list_of_net_assignments COMMA_TOKEN net_assignment
						;

net_assignment : net_lvalue EQU_TOKEN expression
               ;

initial_construct : INITIAL_TOKEN statement { verilogBufferClear(); }
                   | INITIAL_TOKEN error END_TOKEN  { verilogBufferClear();  }
				   ;

always_construct : ALWAYS_TOKEN {
                               currVerilogType=VerilogDocGen::ALWAYS;
                                if(!parseCode){ 
                                 int j=getVerilogToken();
							     if(j!=AT_TOKEN){ // always without ()
							     VerilogDocGen::parseAlways(true);
							     }
							    }
							   } statement {
                                            if(!parseCode && currentFunctionVerilog)
											 {
											  currentFunctionVerilog->endBodyLine=getVerilogEndLine();
											  if( currentFunctionVerilog->endBodyLine<currentFunctionVerilog->startLine || c_lloc.first_line>currentFunctionVerilog->endBodyLine ) // awlays without end
											   currentFunctionVerilog->endBodyLine=c_lloc.first_line;
											  currVerilogType=0;
											  } verilogBufferClear();}
                  | ALWAYS_TOKEN error END_TOKEN { verilogBufferClear();currVerilogType=0;}
				  ; 


blocking_assignment : net_lvalue EQU_TOKEN delay_or_event_control  expression
                    | net_lvalue EQU_TOKEN   expression
					| net_lvalue EQU_TOKEN  delay_or_event_control
				//	| blocking_assignment error SEM_TOKEN
					;

nonblocking_assignment : net_lvalue GT_TOKEN EQU_TOKEN delay_or_event_control  expression
                    | net_lvalue GT_TOKEN EQU_TOKEN   expression
					| net_lvalue GT_TOKEN EQU_TOKEN  delay_or_event_control
					;


procedural_continuous_assignments : ASSIGN_TOKEN variable_assignment
                                   | DEASSIGN_TOKEN net_lvalue
                                   | FORCE_TOKEN net_assignment
                                   | RELEASE_TOKEN net_lvalue
                                   ;

function_blocking_assignment : net_lvalue EQU_TOKEN expression
                             ;


function_statement_or_null : function_statement
                           |  attribute_instance  SEM_TOKEN
						    ;


//------------------------------------------------------------------------------------------------------
//----------------------------A.6.3 Parallel and sequential blocks--------------------------------------------
//-----------------------------------------------------------------------------------------------------

function_seq_block : BEGIN_TOKEN statement_list  END_TOKEN
                   | BEGIN_SPEC_TOKEN block_item_declaration_list  statement_list  END_TOKEN
				   | BEGIN_SPEC_TOKEN  function_statement_list  END_TOKEN
				   | BEGIN_TOKEN block_item_declaration_list  statement_list  END_TOKEN
				   | BEGIN_TOKEN END_TOKEN
				   ;



function_statement_list:function_statement
                       | function_statement_list function_statement
					    ;
						
variable_assignment : net_lvalue EQU_TOKEN expression
                    ;  

par_block : FORK_TOKEN  statement_list  JOIN_TOKEN
          | FORK_SPEC_TOKEN block_item_declaration_list statement_list  JOIN_TOKEN
		  | FORK_SPEC_TOKEN  statement_list  JOIN_TOKEN
		  | FORK_TOKEN JOIN_TOKEN
		  | FORK_TOKEN error JOIN_TOKEN 
		   ;
		   

seq_block : BEGIN_TOKEN  statement_list  END_TOKEN
          | BEGIN_SPEC_TOKEN block_item_declaration_list statement_list  END_TOKEN
		  | BEGIN_SPEC_TOKEN statement_list  END_TOKEN
		  | BEGIN_TOKEN  END_TOKEN
		  | BEGIN_SPEC_TOKEN END_TOKEN
		  | BEGIN_TOKEN error END_TOKEN
		  ;

statement_list:statement
              | statement_list statement
			  ;

statement : attribute_instance blocking_assignment SEM_TOKEN
          | attribute_instance  case_statement
          |  attribute_instance  conditional_statement
          |  attribute_instance  disable_statement
          |  attribute_instance  event_trigger
          |  attribute_instance  loop_statement
          |  attribute_instance  nonblocking_assignment SEM_TOKEN 
          |  attribute_instance  par_block
          |  attribute_instance  procedural_continuous_assignments SEM_TOKEN
          |  attribute_instance  procedural_timing_control_statement 
          |  attribute_instance  seq_block {verilogBufferClear();} 
          |  attribute_instance  system_task_enable
          |  attribute_instance  wait_statement 
		   ;

statement_or_null  : statement
                  |   SEM_TOKEN
               //   | statement error SEM_TOKEN
			      ;
				  


function_statement : attribute_instance  function_blocking_assignment SEM_TOKEN
                   |  attribute_instance  function_case_statement
                   |  attribute_instance  function_conditional_statement
                   |  attribute_instance  function_loop_statement
                   |  attribute_instance  function_seq_block
                   |  attribute_instance  disable_statement
                   |  attribute_instance  system_task_enable
                   ;

function_case_statement:case_statement
                       ;
//------------------------------------------------------------------------------------------------------
//----------------------------A.6.5 Timing control statements--------------------------------------------
//-----------------------------------------------------------------------------------------------------

delay_control: PARA_TOKEN DIGIT_TOKEN {
                                        // identVerilog+=$<cstr>1;
										// writeDigit(); 
										 } 
  	         | PARA_TOKEN LETTER_TOKEN
         	 | PARA_TOKEN RBRACE_TOKEN mintypemax_expression LBRACE_TOKEN {verilogBufferClear();}
             ;

delay_or_event_control : delay_control
                       | event_control 
                       | REPEAT_TOKEN RBRACE_TOKEN expression {currVerilogType=0;}  LBRACE_TOKEN  event_control {currVerilogType=0;}
                       ;

disable_statement : DISABLE_TOKEN identifier SEM_TOKEN
       //           | DISABLE_TOKEN identifier SEM_TOKEN
				  ;

event_control : AT_TOKEN  identifier                                     { VerilogDocGen::parseAlways(); verilogBufferClear();currVerilogType=0;} 
	          | AT_TOKEN RBRACE_TOKEN event_expression_list LBRACE_TOKEN { VerilogDocGen::parseAlways(); verilogBufferClear();currVerilogType=0;}			 
              | AT_TOKEN MULT_TOKEN                                      { VerilogDocGen::parseAlways(); verilogBufferClear();currVerilogType=0;}	
              | AT_TOKEN RBRACE_TOKEN MULT_TOKEN LBRACE_TOKEN            { VerilogDocGen::parseAlways(); verilogBufferClear();currVerilogType=0;}	
			  ;
			  
event_trigger : MINUS_TOKEN LT_TOKEN identifier SEM_TOKEN
              ;

event_expression : expression
                 | POSEDGE_TOKEN expression
                 | NEGEDGE_TOKEN expression
                 ;

lpp:COMMA_TOKEN
   | GATE_TOKEN
  ;


event_expression_list : event_expression 
                      | event_expression_list lpp event_expression 
					  ; 

procedural_timing_control_statement : delay_or_event_control statement_or_null
                                    ;
wait_statement : WAIT_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN statement_or_null

//------------------------------------------------------------------------------------------------------
//----------------------------A.6.6 Conditional statements--------------------------------------------
//-----------------------------------------------------------------------------------------------------

conditional_statement : IF_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN statement_or_null
	                   | IF_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN statement_or_null ELSE_TOKEN statement_or_null
	                   | IF_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN error ELSE_TOKEN statement_or_null
	                  
	                   ;

function_conditional_statement :IF_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN function_statement_or_null
	                           | IF_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN function_statement_or_null ELSE_TOKEN function_statement_or_null
	                          // | IF_TOKEN RBRACE_TOKEN error LBRACE_TOKEN
	                           ;
					  
 


case_statement:	CASE_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN case_item_list ENDCASE_TOKEN
	           |  CASE_TOKEN error ENDCASE_TOKEN { verilogBufferClear();}
			   | CASEZ_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN case_item_list ENDCASE_TOKEN
	           | CASEZ_TOKEN error ENDCASE_TOKEN { verilogBufferClear();}
	           | CASEX_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN case_item_list ENDCASE_TOKEN
	           | CASEX_TOKEN error ENDCASE_TOKEN { verilogBufferClear();}
	           ;


case_item_list: case_item
              | case_item_list case_item
              ;


case_item :expression_list  COLON_TOKEN statement_or_null
	      | DEFAULT_TOKEN COLON_TOKEN  statement_or_null
	      | DEFAULT_TOKEN statement_or_null
          ;

//------------------------------------------------------------------------------------------------------
//----------------------------A.6.8 Looping statements--------------------------------------------
//-----------------------------------------------------------------------------------------------------

function_loop_statement : FOREVER_TOKEN function_statement
                       | REPEAT_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN function_statement
                       | WHILE_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN function_statement
                       | FOR_TOKEN RBRACE_TOKEN variable_assignment SEM_TOKEN expression SEM_TOKEN variable_assignment LBRACE_TOKEN function_statement
                       ;


loop_statement : FOREVER_TOKEN statement
               | REPEAT_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN statement
               | WHILE_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN statement
               | FOR_TOKEN RBRACE_TOKEN variable_assignment SEM_TOKEN expression SEM_TOKEN variable_assignment LBRACE_TOKEN statement
               ;


//------------------------------------------------------------------------------------------------------
//----------------------------A.6.9 Task enable statements--------------------------------------------
//-----------------------------------------------------------------------------------------------------

system_task_enable : simplexx  RBRACE_TOKEN  expression_list  LBRACE_TOKEN SEM_TOKEN
                   | simplexx  RBRACE_TOKEN error  LBRACE_TOKEN SEM_TOKEN
				   | simplexx SEM_TOKEN
				   ;



//------------------------------------------------------------------------------------------------------
//----------------------------A.7.1 Specify block declaration -------------------------------------------
//-----------------------------------------------------------------------------------------------------

specify_block:SPECIFY_TOKEN specify_item_list ENDSPECIFY_TOKEN
             | SPECIFY_TOKEN  ENDSPECIFY_TOKEN { verilogBufferClear(); }
			 | SPECIFY_TOKEN error ENDSPECIFY_TOKEN  { verilogBufferClear(); }
			  ;


specify_item_list:specify_item { verilogBufferClear(); }
              | specify_item_list specify_item { verilogBufferClear(); }
              ;


specify_item: specparam_declaration {verilogBufferClear();}
	        | path_declaration {verilogBufferClear();}
	        | system_timing_check {verilogBufferClear();}
	        | pulsestyle_declaration {verilogBufferClear();}
            | showcancelled_declaration {verilogBufferClear();}
            ;
			

pulsestyle_declaration : PULSEON_DETECT_TOKEN list_of_path_outputs SEM_TOKEN
                       | PULSEONE_EVENT_TOKEN list_of_path_outputs SEM_TOKEN
                       | PULSEONE_EVENT_TOKEN error SEM_TOKEN
					   ;


showcancelled_declaration : SHOWCANCEL_TOKEN list_of_path_outputs SEM_TOKEN
                          | NOSHOWCANCEL_TOKEN  list_of_path_outputs SEM_TOKEN
                          | NOSHOWCANCEL_TOKEN error SEM_TOKEN
						  ;


path_declaration : simple_path_declaration SEM_TOKEN
                 | edge_sensitive_path_declaration SEM_TOKEN
                 | state_dependent_path_declaration SEM_TOKEN
                 ;


simple_path_declaration : parallel_path_description EQU_TOKEN path_delay_value
                       // | full_path_description EQU_TOKEN path_delay_value
                         ;


parallel_path_description : RBRACE_TOKEN specify_input_terminal_descriptor  EQU_TOKEN LT_TOKEN specify_input_terminal_descriptor LBRACE_TOKEN
						  | RBRACE_TOKEN list_of_path_inputs  hit list_of_path_outputs LBRACE_TOKEN
                          | RBRACE_TOKEN list_of_path_inputs    MULT_TOKEN LT_TOKEN list_of_path_outputs LBRACE_TOKEN
					      ;

list_of_path_inputs : specify_input_terminal_descriptor
                    | list_of_path_inputs COMMA_TOKEN specify_input_terminal_descriptor 
                    ;

list_of_path_outputs : list_of_path_inputs 
                     ;
          
hit:polarity_operator  EQU_TOKEN LT_TOKEN
   | polarity_operator  MULT_TOKEN LT_TOKEN 
   ;

//specify_output_terminal_descriptor
//                    | list_of_path_output COMMA_TOKEN specify_output_terminal_descriptor 
//                    ;
//------------------------------------------------------------------------------------------------------
//----------------------------A.7.3 Specify block terminals -------------------------------------------
//-----------------------------------------------------------------------------------------------------

specify_input_terminal_descriptor : identifier
                                  | identifier RBRACKET_TOKEN range_expression LBRACKET_TOKEN 
								  | identifier RBRACKET_TOKEN expression LBRACKET_TOKEN 
								  ;


path_delay_value : list_of_path_delay_expressions
                  | RBRACE_TOKEN list_of_path_delay_expressions LBRACE_TOKEN
                 ;


list_of_path_delay_expressions:path_delay_expression
	            | RBRACE_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression LBRACE_TOKEN
	            | RBRACE_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression LBRACE_TOKEN
	            | RBRACE_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression LBRACE_TOKEN
	            | RBRACE_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression COMMA_TOKEN path_delay_expression LBRACE_TOKEN
	            ;

path_delay_expression:mintypemax_expression


edge_sensitive_path_declaration : parallel_edge_sensitive_path_description EQU_TOKEN path_delay_value
                                | full_edge_sensitive_path_description EQU_TOKEN path_delay_value
                                ;


parallel_edge_sensitive_path_description : RBRACE_TOKEN  edge_identifier  specify_input_terminal_descriptor EQU_TOKEN LT_TOKEN example
                                          |  RBRACE_TOKEN  specify_input_terminal_descriptor EQU_TOKEN LT_TOKEN example                                           
										   ;


full_edge_sensitive_path_description : RBRACE_TOKEN edge_identifier  list_of_path_inputs MULT_TOKEN LT_TOKEN
                                        list_of_path_outputs  pol_mist11  COLON_TOKEN data_source_expression LBRACE_TOKEN
                                     | RBRACE_TOKEN   list_of_path_inputs MULT_TOKEN LT_TOKEN
                                        list_of_path_outputs  pol_mist11  COLON_TOKEN data_source_expression LBRACE_TOKEN
                                      ;



example:   specify_input_terminal_descriptor  pol_mist11  COLON_TOKEN data_source_expression LBRACE_TOKEN
       |  RBRACE_TOKEN specify_input_terminal_descriptor  pol_mist11  COLON_TOKEN data_source_expression LBRACE_TOKEN LBRACE_TOKEN // rule for altera
	   ;                               

pol_mist11:
        | MINUS_TOKEN
        | PLUS_TOKEN
		;  


data_source_expression :  expression
                        ;
edge_identifier :   POSEDGE_TOKEN
	            | NEGEDGE_TOKEN
                ;


state_dependent_path_declaration : IF_TOKEN RBRACE_TOKEN expression LBRACE_TOKEN simple_path_declaration
                                 | IF_TOKEN RBRACE_TOKEN  expression LBRACE_TOKEN edge_sensitive_path_declaration
                                 | IFNONE_TOKEN simple_path_declaration
                                 ;


polarity_operator :  pol_op
                   ;


//------------------------------------------------------------------------------------------------------
//----------------------------A.7.5.1 System timing check commands -------------------------------------------
//-----------------------------------------------------------------------------------------------------


system_timing_check :setup_timing_check
                    | hold_timing_check
                    | setuphold_timing_check
                    | recovery_timing_check
                    | removal_timing_check
                    | skew_timing_check
                    | period_timing_check 
                    | width_timing_check
                    | nochange_timing_check		 
					| recrem_timing_check
                    | fullskew_timing_check 
					| timingskew_timing_check 				    
					;


fullskew_timing_check : FULLSKEW_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list  LBRACE_TOKEN SEM_TOKEN
                     | FULLSKEW_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                     ;

timingskew_timing_check : TIMESKEW_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list  LBRACE_TOKEN SEM_TOKEN
                     | TIMESKEW_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                     ;


recrem_timing_check  : RECREM_TOKEN  RBRACE_TOKEN timing_check_event  COMMA_TOKEN  timing_check_event  COMMA_TOKEN  timing_check_limit  COMMA_TOKEN  timing_check_limit  COMMA_TOKEN notify_register_list LBRACE_TOKEN SEM_TOKEN
                        | RECREM_TOKEN  RBRACE_TOKEN timing_check_event  COMMA_TOKEN  timing_check_event  COMMA_TOKEN  timing_check_limit  COMMA_TOKEN  timing_check_limit  LBRACE_TOKEN SEM_TOKEN
                        ; 
 

setup_timing_check : DSETUP_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list  LBRACE_TOKEN SEM_TOKEN
                   | DSETUP_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                   | DSETUP_TOKEN error SEM_TOKEN
				   ;

hold_timing_check : DHOLD_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list LBRACE_TOKEN SEM_TOKEN
                   | DHOLD_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                   | DHOLD_TOKEN error SEM_TOKEN
				   ;


setuphold_timing_check  : DSETUPHOLD_TOKEN  RBRACE_TOKEN timing_check_event  COMMA_TOKEN  timing_check_event  COMMA_TOKEN  timing_check_limit  COMMA_TOKEN  timing_check_limit  COMMA_TOKEN notify_register_list LBRACE_TOKEN SEM_TOKEN
                        | DSETUPHOLD_TOKEN  RBRACE_TOKEN timing_check_event  COMMA_TOKEN  timing_check_event  COMMA_TOKEN  timing_check_limit  COMMA_TOKEN  timing_check_limit  LBRACE_TOKEN SEM_TOKEN
                        ; 

recovery_timing_check:DRECOVERY_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list  LBRACE_TOKEN SEM_TOKEN
                     | DRECOVERY_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                     ;
removal_timing_check: REMOVAL_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list  LBRACE_TOKEN SEM_TOKEN
                     | REMOVAL_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                     ;


skew_timing_check : DSKEW_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list  LBRACE_TOKEN SEM_TOKEN
                     | DSKEW_TOKEN RBRACE_TOKEN data_event COMMA_TOKEN reference_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                     ;

 nochange_timing_check	:  NOCHANGE_TOKEN  RBRACE_TOKEN timing_check_event COMMA_TOKEN timing_check_event COMMA_TOKEN mintypemax_expression COMMA_TOKEN mintypemax_expression LBRACE_TOKEN SEM_TOKEN
	                    | NOCHANGE_TOKEN   RBRACE_TOKEN timing_check_event COMMA_TOKEN timing_check_event COMMA_TOKEN mintypemax_expression COMMA_TOKEN mintypemax_expression COMMA_TOKEN notify_register_list LBRACE_TOKEN SEM_TOKEN
	                    ;

width_timing_check :  DWIDTH_TOKEN  RBRACE_TOKEN controlled_timing_check_event COMMA_TOKEN timing_check_limit COMMA_TOKEN expression COMMA_TOKEN notify_register_list LBRACE_TOKEN SEM_TOKEN
                     |  DWIDTH_TOKEN  RBRACE_TOKEN controlled_timing_check_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN

					 ;

period_timing_check :DPERIOD_TOKEN   RBRACE_TOKEN controlled_timing_check_event COMMA_TOKEN timing_check_limit LBRACE_TOKEN SEM_TOKEN
                     | DPERIOD_TOKEN  RBRACE_TOKEN controlled_timing_check_event COMMA_TOKEN timing_check_limit COMMA_TOKEN notify_register_list LBRACE_TOKEN SEM_TOKEN
                     ;

notify_register_list:notify_register
                    | notify_register_list COMMA_TOKEN notify_register
					;

 notify_register:expression

                   

//------------------------------------------------------------------------------------------------------
//----------------------------A.7.5.2 System timing check command arguments  -------------------------------------------
//-----------------------------------------------------------------------------------------------------

timing_check_limit :  expression
                   ;
data_event : timing_check_event
           ;

reference_event : timing_check_event
           ;

timing_check_event :timing_check_event_control specify_terminal_descriptor AND_TOKEN AND_TOKEN AND_TOKEN timing_check_condition
                    | timing_check_event_control specify_terminal_descriptor 
                    | specify_terminal_descriptor AND_TOKEN AND_TOKEN AND_TOKEN timing_check_condition
                    | specify_terminal_descriptor 
					 ;

controlled_timing_check_event:timing_check_event
                            
                             ;

timing_check_event_control :POSEDGE_TOKEN
	                       | NEGEDGE_TOKEN
                           | edge_control_specifier
                           ;

edge_control_specifier :EDGE_TOKEN  RBRACKET_TOKEN edge_descriptor_list LBRACKET_TOKEN
                       | error LBRACKET_TOKEN
					   ;

edge_descriptor : DIGIT_TOKEN DIGIT_TOKEN 
	             | DIGIT_TOKEN LETTER_TOKEN
				 | LETTER_TOKEN DIGIT_TOKEN
                 ;

edge_descriptor_list :  edge_descriptor
	             | edge_descriptor_list COMMA_TOKEN edge_descriptor
				 ;

specify_terminal_descriptor : specify_input_terminal_descriptor 
                            ;


timing_check_condition : mintypemax_expression //scalar_timing_check_condition
	                     ;

//------------------------------------------------------------------------------------------------------
//----------------------------A.8.1 Concatenations  -------------------------------------------
//-----------------------------------------------------------------------------------------------------


concatenation : RRAM_TOKEN expression_list LRAM_TOKEN
		                            ;
multiple_concatenation : RRAM_TOKEN expression concatenation LRAM_TOKEN
                        ;
net_concatenation : RRAM_TOKEN net_concatenation_value_list LRAM_TOKEN
                  ;
net_concatenation_value_list: net_concatenation_value
                            | net_concatenation_value_list COMMA_TOKEN net_concatenation_value
							 ;
							 
net_concatenation_value : identifier RBRACKET_TOKEN range_expression LBRACKET_TOKEN
                        | identifier RBRACKET_TOKEN expression LBRACKET_TOKEN
                        | identifier expression_bracket_list range_expression 
                        | net_concatenation
						| identifier						
                        ;
 
expression_bracket_list: RBRACKET_TOKEN expression LBRACKET_TOKEN
                       | expression_bracket_list RBRACKET_TOKEN expression LBRACKET_TOKEN
					    ; 


//------------------------------------------------------------------------------------------------------
//----------------------------A.8.1 Concatenations  -------------------------------------------
//-----------------------------------------------------------------------------------------------------

//genvar_function_call : identifier  attribute_instance RBRACE_TOKEN expression_list LBRACE_TOKEN
//                     ;


//------------------------------------------------------------------------------------------------------
//----------------------------A.8.3 Expressions  -------------------------------------------
//-----------------------------------------------------------------------------------------------------

dimension_constant_expression :expression
                              ;


range_expression :  expression COLON_TOKEN lsb_constant_expression
                 | expression PLUS_TOKEN COLON_TOKEN width_constant_expression
                 | expression MINUS_TOKEN COLON_TOKEN width_constant_expression
                 ;

expression_list: expression
               | expression_list COMMA_TOKEN expression

                          ;
//------------------------------------------------------------------------------------------------------
//----------------------------A.8.4 Primaries  -------------------------------------------
//-----------------------------------------------------------------------------------------------------

function_call :  RBRACE_TOKEN  expression_list LBRACE_TOKEN
			 // | attribute_instance11 RBRACE_TOKEN  expression_list LBRACE_TOKEN
			  ;


primary : simplexx  expression_bracket_list
		| simplexx RBRACKET_TOKEN range_expression LBRACKET_TOKEN 
      	| simplexx
		| simplexx function_call
		| RBRACE_TOKEN mintypemax_expression LBRACE_TOKEN 
        | concatenation
        | multiple_concatenation
		| number
		;


 martin:   primary
           | unary_operator primary
           | unary_operator attribute_instance11 primary
		   | STRING_TOKEN
		   ;

expression : martin 
		   | expression binary_operator  martin 
	       | expression  binary_operator attribute_instance11  martin 
		   | expression QUESTION_TOKEN expression COLON_TOKEN martin //COLON_TOKEN  aaaa
		   ; 

mintypemax_expression : expression
                      | expression COLON_TOKEN expression COLON_TOKEN mintypemax_expression 
                       ;




attribute_instance11 :  RBRACE_TOKEN MULT_TOKEN  attr_spec_list11 MULT_TOKEN LBRACE_TOKEN
                   ;

attr_spec_list11: attr_spec11
                | attr_spec_list11 attr_spec11
			  ; 				   

attr_spec11 : identifier EQU_TOKEN expression
          | identifier
		  ;


lsb_constant_expression :expression
                        ;

msb_constant_expression :expression
                        ; 
                 

width_constant_expression :expression

//------------------------------------------------------------------------------------------------------
//----------------------------A.8.5 Expression left-side values  -------------------------------------------
//-----------------------------------------------------------------------------------------------------

net_lvalue : identifier RBRACKET_TOKEN range_expression LBRACKET_TOKEN
           | identifier expression_bracket_list 
           | net_concatenation
           | simplexx
		   ;


unary_operator :  EXCLAMATION_TOKEN 
			   |  AND_TOKEN
			   |  NOT_TOKEN
			   |  SNAKE_TOKEN
			   |  SNAKE_TOKEN AND_TOKEN
			   |  SNAKE_TOKEN NOT_TOKEN
			   |  OR_TOKEN
			   |  SNAKE_TOKEN OR_TOKEN
			   |  pol_op
			   |  NOT_TOKEN SNAKE_TOKEN 
		        ;


binary_operator : PERCENTAL_TOKEN
				| pol_op // PLUS_TOKEN
				//| MINUS_TOKEN
				| EQU_TOKEN EQU_TOKEN 
			    | EQU_TOKEN EQU_TOKEN EQU_TOKEN
			    | EXCLAMATION_TOKEN EQU_TOKEN
			    | EXCLAMATION_TOKEN EQU_TOKEN EQU_TOKEN
				| AND_TOKEN AND_TOKEN
				| OR_TOKEN OR_TOKEN
		    	| LT_TOKEN
				| GT_TOKEN 
				| GT_TOKEN GT_TOKEN
				| GT_TOKEN EQU_TOKEN
				| LT_TOKEN EQU_TOKEN
				| LT_TOKEN LT_TOKEN
				| MULT_TOKEN
				| ENV_TOKEN
				| NOT_TOKEN
				| AND_TOKEN
				| OR_TOKEN
				| SNAKE_TOKEN
				| SNAKE_TOKEN NOT_TOKEN
	            | NOT_TOKEN SNAKE_TOKEN 
                | MULT_TOKEN MULT_TOKEN      // vlog 2001
				| GT_TOKEN GT_TOKEN GT_TOKEN // vlog2001
				| LT_TOKEN LT_TOKEN LT_TOKEN // vlog2001
                ;


 signed :  SIGNED_TOKEN
		;

pol_op:MINUS_TOKEN
      | PLUS_TOKEN
                 ;
				  

number : DIGIT_TOKEN { identVerilog+=$<cstr>1;writeDigit(); } 
	     | pol_op DIGIT_TOKEN  {if(parseCode) {writePrevVerilogWords(identVerilog);writeVerilogFont("vhdllogic",identVerilog.data());identVerilog.resize(0);}}
      ;



//------------------------------------------------------------------------------------------------------
//----------------------------A.9.1 Attributes  -------------------------------------------
//-----------------------------------------------------------------------------------------------------

attribute_instance : /* empty */
                   |RBRACE_TOKEN MULT_TOKEN  attr_spec_list MULT_TOKEN LBRACE_TOKEN
                   ;

attr_spec_list: attr_spec
              | attr_spec_list COMMA_TOKEN attr_spec
			  ;				   

attr_spec : identifier EQU_TOKEN identifier
          | identifier EQU_TOKEN DIGIT_TOKEN 
		  | identifier
		  ;

 simplexx : identifier 
          | simplexx DOT_TOKEN identifier	{
										 }               		  
		  ;

identifier:ident { parseString(); }


ident : LETTER_TOKEN  {
                    	if(parseCode) 
						      identVerilog+=$<cstr>1; 
							  yydebug=0;
							 } //LETTER_TOKEN 
        | DOLLAR_TOKEN  ident {}  	
	  ;	





%%
//------ ------------------------------------------------------------------------------------------------

Entry* getCurrVerilogEntry(){return currentVerilog;}

Entry* VerilogDocGen::getCurrBrief(){return &prevDocEntryVerilog; }

void setGlobalVerilogEntry(Entry* ee,bool pc){
 parseCode=pc;
if(pc) return;
  current_rootVerilog=ee;
  currentVerilog=0; 
  current_rootVerilog->name=QCString("XXX"); // dummy name for root
}


 Entry* VerilogDocGen::makeNewEntry(char* name,int sec,int spec,int line){
 
  Entry *e=new Entry;
  assert(e);
  if(line){
  	  e->bodyLine=line;
     e->startLine=line;
  }else
   {
     e->bodyLine=getVerilogPrevLine();
     e->startLine=getVerilogPrevLine();
   }
  e->section=sec;
  e->spec=spec;
  e->name=name;
  return e;
 }


void VerilogDocGen::setBrief(QCString& text,int line)
{
 if(prevDocEntryVerilog.brief.isEmpty())
  prevDocEntryVerilog.briefLine=line;
 else
   prevDocEntryVerilog.brief+='\n';
 
 prevDocEntryVerilog.brief+=text;
 
 isPrevDocVerilog=true;
}

void VerilogDocGen::addSubEntry(Entry* root, Entry* e) {
 if(e==NULL || root==NULL) return;
 if(isPrevDocVerilog) {
  
  if(prevDocEntryVerilog.brief.contains("//%")>1) {
   e->doc=prevDocEntryVerilog.brief;
   e->docLine=prevDocEntryVerilog.briefLine;
   }
   else {
   e->brief=prevDocEntryVerilog.brief;
   e->briefLine=prevDocEntryVerilog.briefLine;
   }
   prevDocEntryVerilog.reset();
   isPrevDocVerilog=false;
  }

 root->addSubEntry(e);

} 

QCString VerilogDocGen::convertTypeToString(int type)
{
  uint ttype=(uint)type;

  if(VerilogDocGen::MODULE==ttype)  return "Module"; 
  if( VerilogDocGen::FUNCTION==ttype) return "Functions";
  if(VerilogDocGen::WIRE==ttype) return "Wire";
  if(VerilogDocGen::REGISTER==ttype) return "Registers";
  if(VerilogDocGen::PRIMITIVE==ttype)  return "Primitive";
  if(VerilogDocGen::INTEGER==ttype) return "Integers/Reals";
  if(ttype==VerilogDocGen::PARAMETER)  return "Parameters";
  if(VerilogDocGen::COMPONENT==ttype) return "Components";
  if( VerilogDocGen::PORT==ttype) return "Ports";
  if( VerilogDocGen::ALWAYS==ttype) return "Processes";
  if( VerilogDocGen::TRI==ttype) return "Tri";
  if( VerilogDocGen::INIT==ttype) return "Init";
  if( VerilogDocGen::INPUT==ttype) return "Inputs";
  if( VerilogDocGen::OUTPUT==ttype) return "Outputs";
  if( VerilogDocGen::INOUT==ttype) return "Inouts";
  if( VerilogDocGen::NETTYPE==ttype) return "Nettype";  
  if( VerilogDocGen::FEATURE==ttype) return "Defines"; 
  if( VerilogDocGen::TIME==ttype) return "Time"; 
  if( VerilogDocGen::INCLUDE==ttype) return "Includes"; 
  return "";
 
} // convertType

void VerilogDocGen::writeVHDLDeclarations(MemberList* ml,OutputList &ol,
               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
               const char *title,const char *subtitle,bool showEnumValues,int type) {


  MemberDef *mdd=NULL;
  MemberListIterator mmli(*ml);

  bool hasType=false;
  for ( ; (mdd=mmli.current()); ++mmli )
  {
	  int mems=mdd->getMemberSpecifiers();
      if(mdd->getMemberSpecifiers()==type)
      {
		  hasType=true;
		  break;
	  }
  }

  if(!hasType) return;
  
  if (title) 
  {
    ol.startMemberHeader();
    ol.parseText(title);
    ol.endMemberHeader();
	ol.docify(" ");
  }
  if (subtitle && subtitle[0]!=0) 
  {
    //printf("subtitle=`%s'\n",subtitle);
    ol.startMemberSubtitle();
    ol.parseDoc("[generated]",-1,0,0,subtitle,FALSE,FALSE);
    ol.endMemberSubtitle();
  } 
  
  VerilogDocGen::writePlainVHDLDeclarations(mdd,ml,ol,cd,nd,fd,gd,type);
  
  if (ml->getMemberGroupList())
  {
    assert(0);
    MemberGroupListIterator mgli(*ml->getMemberGroupList());
    MemberGroup *mg;
    while ((mg=mgli.current()))
    {
      //printf("mg->header=%s\n",mg->header().data());
      bool hasHeader=mg->header()!="[NOHEADER]";
      ol.startMemberGroupHeader(hasHeader);
      if (hasHeader)
      {
        ol.parseText(mg->header());
      }
      ol.endMemberGroupHeader();
      if (!mg->documentation().isEmpty())
      {
        //printf("Member group has docs!\n");
        ol.startMemberGroupDocs();
        ol.parseDoc("[generated]",-1,0,0,mg->documentation()+"\n",FALSE,FALSE);
        ol.endMemberGroupDocs();
      }
      ol.startMemberGroup();
      //printf("--- mg->writePlainDeclarations ---\n");
      mg->writePlainDeclarations(ol,cd,nd,fd,gd);
      ol.endMemberGroup(hasHeader);
      ++mgli;
    }
  }
 }// writeVerilogDeclarations


void VerilogDocGen::writePlainVHDLDeclarations(MemberDef* mdef,MemberList* mlist,OutputList &ol,
               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,int specifier){

  
  ol.pushGeneratorState();

  bool first=TRUE;
  MemberDef *md;
  MemberListIterator mli(*mlist);
  for ( ; (md=mli.current()); ++mli )
  { 
	int mems=md->getMemberSpecifiers();
	 
    if (md->isBriefSectionVisible() && (mems==specifier))
    {
             if (first) ol.startMemberList(),first=FALSE;
			VerilogDocGen::writeVHDLDeclaration(md,ol,cd,nd,fd,gd,false);
    }//if
  }//for
  if (!first) ol.endMemberList(); 
  
}//plainDeclaration

void VerilogDocGen::writeVhdlDeclarations(MemberList* ml,OutputList& ol,GroupDef* gd,ClassDef* cd){

	  static ClassDef *cdef;
	  static GroupDef* gdef;
	 
	  if(cd && cdef!=cd) { // only one inline link
	  //VerilogDocGen::writeInlineClassLink(cd,ol);
	  cdef=cd;
	  }
	  
	  if(gd && gdef==gd) return;
	  if(gd && gdef!=gd)
	   gdef=gd;
	  
	  VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::PORT),0,FALSE,VerilogDocGen::PORT); 
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::MODULE),0,FALSE,VerilogDocGen::MODULE);
     VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::FEATURE),0,FALSE,VerilogDocGen::FEATURE);
    
	 VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::INCLUDE),0,FALSE,VerilogDocGen::INCLUDE);
    
	  VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::FUNCTION),0,FALSE,VerilogDocGen::FUNCTION);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::INIT),0,FALSE,VerilogDocGen::INIT);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::WIRE),0,FALSE,VerilogDocGen::WIRE);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::INTEGER),0,FALSE,VerilogDocGen::INTEGER);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::TIME),0,FALSE,VerilogDocGen::TIME);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::ALWAYS),0,FALSE,VerilogDocGen::ALWAYS);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::TRI),0,FALSE,VerilogDocGen::TRI);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::INPUT),0,FALSE,VerilogDocGen::INPUT);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::INOUT),0,FALSE,VerilogDocGen::INOUT);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::OUTPUT),0,FALSE,VerilogDocGen::OUTPUT);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::PARAMETER),0,FALSE,VerilogDocGen::PARAMETER);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::REGISTER),0,FALSE,VerilogDocGen::REGISTER);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::NETTYPE),0,FALSE,VerilogDocGen::NETTYPE);
      VerilogDocGen::writeVHDLDeclarations(ml,ol,cd,0,0,gd,VerilogDocGen::convertTypeToString(VerilogDocGen::COMPONENT),0,FALSE,VerilogDocGen::COMPONENT);
 
 	 }


void VerilogDocGen::writeVHDLDeclaration(MemberDef* mdef,OutputList &ol,
                   ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
                   bool inGroup) {
 
  static bool bComp=false;
  LockingPtr<MemberDef> lock(mdef,mdef);
 
  Definition *d=0;
  ASSERT (cd!=0 || nd!=0 || fd!=0 || gd!=0); // member should belong to something
 // if (cd) d=cd; else if (nd) d=nd; else if (fd) d=fd; else d=gd;
if (cd) d=cd;
  // write tag file information of this member
 int memType=mdef->getMemberSpecifiers();

  if (!Config_getString("GENERATE_TAGFILE").isEmpty())
  {
    Doxygen::tagFile << "    <member kind=\"";
    Doxygen::tagFile << VerilogDocGen::convertTypeToString(memType);
     
    Doxygen::tagFile << "\">" << endl;
    Doxygen::tagFile << "      <type>" << convertToXML(mdef->typeString()) << "</type>" << endl;
    Doxygen::tagFile << "      <name>" << convertToXML(mdef->name()) << "</name>" << endl;
    Doxygen::tagFile << "      <anchorfile>" << convertToXML(mdef->getOutputFileBase()+Doxygen::htmlFileExtension) << "</anchorfile>" << endl;
    Doxygen::tagFile << "      <anchor>" << convertToXML(mdef->anchor()) << "</anchor>" << endl;
  
	if(memType==VerilogDocGen::FUNCTION)
		Doxygen::tagFile << "      <arglist>" << convertToXML(VhdlDocGen::convertArgumentListToString(mdef->argumentList().pointer(),true)) << "</arglist>" << endl;
    else if(memType==VerilogDocGen::ALWAYS)
		Doxygen::tagFile << "      <arglist>" << convertToXML(VhdlDocGen::convertArgumentListToString(mdef->argumentList().pointer(),false)) << "</arglist>" << endl;
	else{
	Doxygen::tagFile << "      <arglist>" << convertToXML(mdef->argsString()) << "</arglist>" << endl;
   Doxygen::tagFile << "      <arglist>" << convertToXML(mdef->typeString()) << "</arglist>" << endl; 
    }
	mdef->writeDocAnchorsToTagFile();
    Doxygen::tagFile << "    </member>" << endl;
 
  }
  
  // write search index info
  if (Config_getBool("SEARCHENGINE"))
  {
    Doxygen::searchIndex->setCurrentDoc(mdef->qualifiedName(),mdef->getOutputFileBase(),mdef->anchor());
    Doxygen::searchIndex->addWord(mdef->localName(),TRUE);
    Doxygen::searchIndex->addWord(mdef->qualifiedName(),FALSE);
  }

  QCString cname  = d->name();
  QCString cfname = mdef->getOutputFileBase();

 // HtmlHelp *htmlHelp=0;
//  bool hasHtmlHelp = Config_getBool("GENERATE_HTML") && Config_getBool("GENERATE_HTMLHELP");
//  if (hasHtmlHelp) htmlHelp = HtmlHelp::getInstance();

  // search for the last anonymous scope in the member type
  ClassDef *annoClassDef=mdef->getClassDefOfAnonymousType();

  // start a new member declaration
  bool isAnonymous = annoClassDef; // || m_impl->annMemb || m_impl->annEnumType;
  ///printf("startMemberItem for %s\n",name().data());
  ol.startMemberItem( isAnonymous ); //? 1 : m_impl->tArgList ? 3 : 0);


  // If there is no detailed description we need to write the anchor here.
  bool detailsVisible = mdef->isDetailedSectionLinkable();
  if (!detailsVisible) // && !m_impl->annMemb)
  {
     QCString doxyName=mdef->name().copy();
    if (!cname.isEmpty()) doxyName.prepend(cname+"::");
    QCString doxyArgs=mdef->argsString();
    ol.startDoxyAnchor(cfname,cname,mdef->anchor(),doxyName,doxyArgs);

    ol.pushGeneratorState();
    ol.disable(OutputGenerator::Man);
    ol.disable(OutputGenerator::Latex);
    ol.docify("\n");
    ol.popGeneratorState();
    
  }
// *** write type
     /*VHDL CHANGE */
   QCString ltype(mdef->typeString()); 
   QCString largs(mdef->argsString());
   VhdlDocGen::adjustRecordMember(mdef); 
   int mm=mdef->getMemberSpecifiers();

   ClassDef *kl=NULL;
   FileDef *fdd=NULL;
   LockingPtr<ArgumentList> alp = mdef->argumentList();
   QCString nn;
   if(gd)gd=NULL;
   switch(mm)
   {
   case VerilogDocGen::INCLUDE: 
       VhdlDocGen::writeLink(mdef,ol); 
        break;
	case VerilogDocGen::FEATURE: 
       	ol.startBold();
        VhdlDocGen::formatString(ltype,ol,mdef);
        ol.docify(" ");
		VhdlDocGen::writeLink(mdef,ol);
	    ol.insertMemberAlign();
		VhdlDocGen::formatString(largs,ol,mdef);
        ol.endBold();
		break;

	case VerilogDocGen::MODULE: 
       	ol.startBold();
        VhdlDocGen::formatString(ltype,ol,mdef);
        ol.endBold();
		ol.insertMemberAlign();
	   //writeLink(mdef,ol);
    case VerilogDocGen::PORT:
    case VerilogDocGen::TIME:
	case VerilogDocGen::INTEGER:      
		  VhdlDocGen::writeLink(mdef,ol);
		 ol.insertMemberAlign();
		  if(largs.length()>0)
		    VhdlDocGen::formatString(largs,ol,mdef);
          if(ltype.length()>0)
		    VhdlDocGen::formatString(ltype,ol,mdef);	  
		  break;
    case VerilogDocGen::ALWAYS:
	     VhdlDocGen::writeLink(mdef,ol);  
	     ol.insertMemberAlign();
		 VhdlDocGen::writeProcessProto(ol,alp.pointer(),mdef);
		break;
   case VerilogDocGen::FUNCTION:
         VhdlDocGen::writeLink(mdef,ol);  
	   	 ol.insertMemberAlign();
		 if(ltype.length()>0)
		    VhdlDocGen::formatString(ltype,ol,mdef);
	 	 VhdlDocGen::writeFunctionProto(ol,alp.pointer(),mdef);
		 break;
   case VerilogDocGen::REGISTER:
         if(ltype.length()>0 && largs.length()>0 )
		   VhdlDocGen::formatString(ltype,ol,mdef);
            VhdlDocGen::writeLink(mdef,ol);  
            ol.insertMemberAlign();
            ol.docify(" ");
	   	if(ltype.length()>0 && largs.isEmpty() )
		   VhdlDocGen::formatString(ltype,ol,mdef);
	   	   
           if(largs.length()>0)
		    VhdlDocGen::formatString(largs,ol,mdef);
        break;
   case VerilogDocGen::INPUT:
   case VerilogDocGen::OUTPUT:
   case VerilogDocGen::INOUT:
   case VerilogDocGen::PARAMETER:   	    
         VhdlDocGen::writeLink(mdef,ol);  
	   	 ol.insertMemberAlign();
		 if(ltype.length()>0){
		    VhdlDocGen::formatString(ltype,ol,mdef);
	   	   ol.docify("  ");
	    }
	  if(largs.length()>0)
		    VhdlDocGen::formatString(largs,ol,mdef);	 
	 	 break;
  
  case VerilogDocGen::NETTYPE:   	    
		 VhdlDocGen::writeLink(mdef,ol);  
	   	 ol.insertMemberAlign();
	     if(ltype.length()>0){
		    VhdlDocGen::formatString(ltype,ol,mdef);
	 	  //  ol.insertMemberAlign();
		 }
		 ol.docify(" ");
		 ol.startBold();
		 ol.docify(largs.data());
		 ol.endBold();
		 break;
    case VerilogDocGen::COMPONENT:
		 VhdlDocGen::writeLink(mdef,ol);
           
        if(true)
		{
		nn=mdef->name().lower();
		kl=getClass(ltype);
		ol.insertMemberAlign();
	  if(kl==NULL){
	    ol.startBold();
	    ol.docify(ltype);
        ol.docify("  ");
        ol.endBold();
       }
		if(kl) {
			nn=kl->getOutputFileBase();
		
		ol.pushGeneratorState();
        ol.disableAllBut(OutputGenerator::Html);
         ol.docify("   ");
        QCString name=VerilogDocGen::getClassTitle(kl);
	     name=VhdlDocGen::getIndexWord(name.data(),1);
	    // ol.insertMemberAlign();
	 
	     ol.startBold();
		ol.docify(name.data());
		ol.endBold();
		ol.startEmphasis();
		   ol.docify(" ");
      
	    ol.writeObjectLink(kl->getReference(),kl->getOutputFileBase(),0,ltype);
	    ol.endEmphasis();
        ol.popGeneratorState();
		}
        } 
		break;
  default: break;
   }

   bool htmlOn = ol.isEnabled(OutputGenerator::Html);
  if (htmlOn && Config_getBool("HTML_ALIGN_MEMBERS") && !ltype.isEmpty())
  {
    ol.disable(OutputGenerator::Html);
  }
  if (!ltype.isEmpty()) ol.docify(" ");
  
  if (htmlOn) 
  {
    ol.enable(OutputGenerator::Html);
  }

  if (!detailsVisible)// && !m_impl->annMemb)
  {
    ol.endDoxyAnchor(cfname,mdef->anchor());
  }

  //printf("endMember %s annoClassDef=%p annEnumType=%p\n",
  //    name().data(),annoClassDef,annEnumType);
  ol.endMemberItem();
   if (!mdef->briefDescription().isEmpty() &&   Config_getBool("BRIEF_MEMBER_DESC") /* && !annMemb */)
  {
    ol.startMemberDescription();
    ol.parseDoc(mdef->briefFile(),mdef->briefLine(),mdef->getOuterScope()?mdef->getOuterScope():d,mdef,mdef->briefDescription(),TRUE,FALSE);
    if (detailsVisible) 
    {
      ol.pushGeneratorState();
      ol.disableAllBut(OutputGenerator::Html);
      //ol.endEmphasis();
      ol.docify(" ");
      if (mdef->getGroupDef()!=0 && gd==0) // forward link to the group
      {
        ol.startTextLink(mdef->getOutputFileBase(),mdef->anchor());
      }
      else // local link
      {
        ol.startTextLink(0,mdef->anchor());
      }
      ol.endTextLink();
      //ol.startEmphasis();
      ol.popGeneratorState();
    }
    //ol.newParagraph();

    ol.endMemberDescription();
     if(VhdlDocGen::isComponent(mdef))
      ol.lineBreak();
  }
   mdef->warnIfUndocumented();

  }// end writeVerilogDeclaration

//-------------------------------------------------------------------------

// extracts module/primitive name

void VerilogDocGen::parseModule(const char* s){
 
 QCString mod(getVerilogString());
 VhdlDocGen::deleteAllChars(mod,'\n');
 VhdlDocGen::deleteAllChars(mod,' ');
 VhdlDocGen::deleteAllChars(mod,';');
 VhdlDocGen::deleteAllChars(mod,'(');
 
 QRegExp reg("[^_a-zA-Z0-9$]");

 int ll=mod.find(reg);

 if(ll>-1){
  char c=mod.at(ll);
  QCString val=mod.remove(ll,1);

 }
assert(mod.length()<100);

 if(parseCode) {
 //generateVerilogClassOrGlobalLink(mod.data());
 currVerilogClass=mod;
 return;
 }
  currentVerilog->name=mod;
 }//parseModuleName


// extracts module instances [ module_name name,module_name #(...) name]

void VerilogDocGen::parseModuleInst(QCString& first, QCString& sec) {
 
if(currVerilogType==VerilogDocGen::DEFPARAM) return;

 VhdlDocGen::deleteAllChars(sec,'(');
 VhdlDocGen::deleteAllChars(sec,'\n');
 VhdlDocGen::deleteAllChars(sec,')');
 VhdlDocGen::deleteAllChars(sec,' ');
 VhdlDocGen::deleteAllChars(sec,',');
 VhdlDocGen::deleteAllChars(sec,';');
 QCString temp=sec;
//while(sec.stripPrefix(" "));

if(sec!=first && (sec.contains("#")==0))
{ 
 //QStringList ql=QStringList::split(first.data(),sec,false);
int oo=sec.findRev(first.data());
if(oo>0) 
 sec=sec.left(oo);
}
else
 sec=getLastLetter();

if(temp.contains("#"))
{ 
 int ii=temp.find("#");
 sec=temp.left(ii);
while(sec.stripPrefix(" "));
}

 if(parseCode){
   currVerilogInst=sec;
   return;
  }
 else {
  Entry* pTemp=VerilogDocGen::makeNewEntry(first.data(),Entry::VARIABLE_SEC,VerilogDocGen::COMPONENT,c_lloc.first_line);
  pTemp->type=sec;
 
  VerilogDocGen::addSubEntry(currentVerilog,pTemp);	
 if(sec==first)return;
 if(!findExtendsComponent(currentVerilog->extends,sec)){	
  	BaseInfo *bb=new BaseInfo(sec,Private,Normal);
    currentVerilog->extends->append(bb);						
   }
  }
}


void VerilogDocGen::parseListOfPorts(const char* s) {
 
  QCString type;

 QCString mod(getVerilogString());
 
 VhdlDocGen::deleteAllChars(mod,' ');
 VhdlDocGen::deleteAllChars(mod,'\n');
 VhdlDocGen::deleteAllChars(mod,';');
 VhdlDocGen::deleteAllChars(mod,')');
 VhdlDocGen::deleteAllChars(mod,'(');
  QStringList ql=QStringList::split(",",mod,false);
  QCString name=(QCString)ql[0];
if(!parseCode) {
  for(uint j=0;j<ql.count();j++) {
  QCString name=(QCString)ql[j];
   int i=name.find('[');
  if(i > 0){
    type=mod.right(mod.length()-i);
    name=mod.left(i);
  }
  
 name.prepend(VhdlDocGen::getRecordNumber().data());
 Entry* pTemp=VerilogDocGen::makeNewEntry(name.data(),Entry::VARIABLE_SEC,VerilogDocGen::PORT,c_lloc.first_line);
  pTemp->type=type; 
  VerilogDocGen::addSubEntry(currentVerilog,pTemp);			          	
   }
  return;
 }	


  //generateVerilogMemLink(currVerilogClass,name,VerilogDocGen::PORT);

 }//parseListOfPorts



void VerilogDocGen::parseReg(Entry* e,const char* s){

static QCString prevType;
QCString regType;
QCString qcs;

 if((CurrState==VerilogDocGen::STATE_FUNCTION || CurrState==VerilogDocGen::STATE_TASK )) return;

QCString mod(getVerilogString());
int porttype;
 VhdlDocGen::deleteAllChars(mod,'(');
  VhdlDocGen::deleteAllChars(mod,')');
VhdlDocGen::deleteAllChars(mod,';');
VhdlDocGen::deleteAllChars(mod,'\n');

if(mod.contains("="))
{
 int i=mod.find("=");
 qcs=mod.right(mod.length()-i-1);
 VhdlDocGen::deleteAllChars(qcs,' ');
 mod=mod.left(i);
}

 while(mod.stripPrefix(" "));

if(mod.stripPrefix("reg"))
 porttype=1;
else if(mod.stripPrefix("integer"))
 porttype=2;
else if(mod.stripPrefix("real"))
 porttype=3;
else if(mod.stripPrefix("time"))
 porttype=4;
else if(mod.stripPrefix("realtime"))
 porttype=4;

 
if(porttype==1){
  while(mod.stripPrefix(" "));
  if(mod.stripPrefix("signed"))
   prevType+="signed ";
}

VhdlDocGen::deleteAllChars(mod,' ');


  int i=mod.find(']');
  int h=mod.find('[');

  if(h==0){
  	prevType+=mod.left(i+1);
  	mod=mod.right(mod.length()-i-1);
  h=mod.find('[');
  }

  if(h > 0){
   if(porttype!=2){ 
	regType=mod.right(mod.length()-h);
    mod=mod.left(h);
   }
   else {
    int ii=mod.find('[');
	if(ii>0){
  	prevType=mod.mid(ii,mod.length());
   	mod=mod.left(ii);
   }
  }
 }

  QStringList ql=QStringList::split(",",mod,false);
  for(uint j=0;j<ql.count();j++) {
  QCString name=(QCString)ql[j];
  name.prepend(VhdlDocGen::getRecordNumber().data());
 
  Entry* pTemp=VerilogDocGen::makeNewEntry(name.data(),Entry::VARIABLE_SEC,getVerilogPrevLine());
  pTemp->type=prevType;
  pTemp->args=regType;
  pTemp->args+=qcs;
 
  pTemp->spec=currVerilogType;
  
  regType=prevType;
  if(getVerilogToken()==SEM_TOKEN) { prevType="";}
  
  VerilogDocGen::addSubEntry(e,pTemp);	
 }
} // parsReg


// extracts function/task prototype 

void VerilogDocGen::parseFunction(Entry* curF,const char *s)
{
  QCString mod(getVerilogString());
  QCString type; 
 
 VhdlDocGen::deleteAllChars(mod,'\n');
 VhdlDocGen::deleteAllChars(mod,';');
  while(mod.stripPrefix(" "));
 
  int i=mod.findRev(']');
  if(i > 0){
    type=mod.left(i+1);
   	mod=mod.right(mod.length()-i-1);
  }
  else {
  QStringList ql=QStringList::split(" ",mod,false);
  if(ql.count()>1) {
    type=(QCString)ql[0];
	mod=(QCString)ql[1];
  }
  }

 VhdlDocGen::deleteAllChars(mod,' ');
 VhdlDocGen::deleteAllChars(type,' ');

  curF->name=mod;
  curF->type=type;
}
							   

// extract (local)parameter declaration 

void VerilogDocGen::parseParam(Entry* e,const char* s)
{
   QCString prevType,qcs;
  QRegExp regg("[ \t]");

  if((CurrState==VerilogDocGen::STATE_FUNCTION || CurrState==VerilogDocGen::STATE_TASK )) return;
  
  QCString mod(getVerilogString());
  VhdlDocGen::deleteAllChars(mod,';');
  VhdlDocGen::deleteAllChars(mod,'\n');
  VhdlDocGen::deleteAllChars(mod,',');

if(mod.contains("="))
{
 int i=mod.find("=");
 qcs=mod.right(mod.length()-i-1);
 while(qcs.stripPrefix(" "));
 mod=mod.left(i);
}

 while(mod.stripPrefix(" "));

 int j=mod.find(regg,0);
			 if(j>0){
			 bool bb=false;
			 QCString sem=mod.mid(0,j);
			 if(sem=="integer"){ prevType=sem;bb=true;}
			 else if(sem=="real"){prevType=sem;bb=true;}
			 else if(sem=="realtime"){prevType=sem;bb=true;}
			 else if(sem=="time"){prevType=sem;bb=true;}
			 else if(sem=="signed"){prevType=sem;bb=true;}
			 else if(sem=="wire"){prevType=sem;bb=true;}
			 if(bb)
			 mod.stripPrefix(sem.data());
			 }

 
 while(mod.stripPrefix(" "));
  
  int i=mod.find(']');
  if(i > 0){
    prevType+=" ";
	prevType+=mod.left(i+1);
   	mod=mod.right(mod.length()-i-1);
  }
 

  VhdlDocGen::deleteAllChars(mod,' ');


// each local member must get its unique number, because in Verilog
// two local variables can have the same identifier.
// ( input Q, reg Q)
  mod.prepend(VhdlDocGen::getRecordNumber().data());
 
  Entry* pTemp=VerilogDocGen::makeNewEntry(mod.data(),Entry::VARIABLE_SEC,VerilogDocGen::PARAMETER,getVerilogPrevLine());
  pTemp->type=prevType;
  pTemp->args=qcs;
  
  VerilogDocGen::addSubEntry(e,pTemp);	
 
}

// extract  input/output ports

void VerilogDocGen::parsePortDir(Entry* e,int port,const char* s)
{

static QCString prevType;
static QCString type; 
static int porttype;

bool sem=false;

QCString mod(getVerilogString());
 VhdlDocGen::deleteAllChars(mod,'\n');
 VhdlDocGen::deleteAllChars(mod,')');
 VhdlDocGen::deleteAllChars(mod,'(');
 VhdlDocGen::deleteAllChars(mod,';');
 VhdlDocGen::deleteAllChars(mod,',');

 while(mod.stripPrefix(" "));


if(mod.stripPrefix("input"))
 prevType="";
else if(mod.stripPrefix("output"))
  prevType=""; 
else if(mod.stripPrefix("inout"))
  prevType="";
else {
             QRegExp regg("[ \\[]");
  			 int j=mod.find(regg,0);
			 if(j>0){
			 type=mod.mid(0,j);
			 mod.stripPrefix(type.data());
			 }
	 } 


while(mod.stripPrefix(" "));
 
QRegExp regg("[ \t]");
 int j=mod.find(regg,0);
			 if(j>0){
			 bool bb=false;
			 QCString sem=mod.mid(0,j);
			 if(sem=="integer"){ prevType=sem;bb=true;}
			 else if(sem=="real"){prevType=sem;bb=true;}
			 else if(sem=="realtime"){prevType=sem;bb=true;}
			 else if(sem=="time"){prevType=sem;bb=true;}
			 else if(sem=="signed"){prevType=sem;bb=true;}
			 else if(sem=="wire"){prevType=sem;bb=true;}
			 if(bb)
			 mod.stripPrefix(sem.data());
			 }

while(mod.stripPrefix(" "));
  
  int i=mod.findRev(']');
  if(i > 0){
    prevType+=" ";
	prevType+=mod.left(i+1);
   	mod=mod.right(mod.length()-i-1);
  }
  else{ 
  int j=mod.find(regg,0);
   if(j>0){
    QCString sem=mod.mid(0,j);
	if(sem=="reg"){		
     mod=mod.right(mod.length()-j-1);
     prevType+=" reg";
     }
    }
   }
  
  VhdlDocGen::deleteAllChars(mod,' ');
   mod.prepend(VhdlDocGen::getRecordNumber().data());
  Entry* pTemp=VerilogDocGen::makeNewEntry(mod.data(),Entry::VARIABLE_SEC,0,c_lloc.first_line);
  pTemp->type=prevType;
  pTemp->args=type;
   

assert(currVerilogType!=0);

pTemp->spec=currVerilogType;

  if(CurrState==VerilogDocGen::STATE_MODULE)
   VerilogDocGen::addSubEntry(currentVerilog,pTemp);
  else if(CurrState==VerilogDocGen::STATE_FUNCTION){
      Argument *arg=new Argument;
      
      switch(currVerilogType) {
      
      case VerilogDocGen::INPUT: arg->type="Input";break;
      case VerilogDocGen::INOUT:arg->type="Inout";break;         
      case VerilogDocGen::OUTPUT:arg->type="Output";break;         
      default:break;
      }                           
        arg->defval=prevType;                         
        arg->name=mod;//(QCString)ql[j];	
	  currentFunctionVerilog->argList->append(arg);
	  VhdlDocGen::adjustMemberName(mod); 
	  currentFunctionVerilog->args+=mod;//(QCString)ql[j]+",";
	  delete pTemp;
  } 
 //}
 

  if(getVerilogToken()==SEM_TOKEN)//end of line
  {prevType="";type="";}
}

void VerilogDocGen::parseAlways(bool bBody)
{

if(currVerilogType!=VerilogDocGen::ALWAYS) return ;

QRegExp regg1("[ \t]or[ \t]");

QCString mod(getVerilogString());
QCString type; 
QStringList ql;
bool sem=false;

 VhdlDocGen::deleteAllChars(mod,'@');
 VhdlDocGen::deleteAllChars(mod,'\n');
 VhdlDocGen::deleteAllChars(mod,'(');
 VhdlDocGen::deleteAllChars(mod,')');
 
if(mod.contains(","))
  ql=QStringList::split(",",mod,false);
 else
  ql=QStringList::split(regg1,mod,false);
 

 if(!parseCode) {
 currentFunctionVerilog=VerilogDocGen::makeNewEntry(VhdlDocGen::getProcessNumber().data(),Entry::FUNCTION_SEC,VerilogDocGen::ALWAYS);
  currentFunctionVerilog->stat=TRUE;
  currentFunctionVerilog->fileName=getVerilogParsingFile();
  if(!bBody)
  for(uint j=0;j<ql.count();j++) {
  QCString ll=(QCString)ql[j];
  if(ll=="or" || ll=="and") continue; 
  if(sem)
	  currentFunctionVerilog->args+=',';
	  Argument *arg=new Argument;
      arg->name=ll;	
	  currentFunctionVerilog->argList->append(arg);
      currentFunctionVerilog->args+=ll; 
      sem = true;
 }
 VerilogDocGen::addSubEntry(currentVerilog,currentFunctionVerilog);
 return;
}


}//parseAlways



//-----------------< Code Parsing >------------------------------------------------


MemberDef* VerilogDocGen::findMember(QCString& className, QCString& memName,int type)
{
	ClassDef* cd;
	MemberDef *mdef=NULL;
	bool feat=false;


	cd=getClass(className.data());
    if(!cd) return NULL;

	 mdef=VerilogDocGen::findMemberDef(cd,memName,MemberList::variableMembers,type,feat);
     if(mdef) return mdef;
  	
   if(memName.contains('`')){
     memName.stripPrefix("`");
     mdef=VerilogDocGen::findMemberDef(cd,memName,MemberList::variableMembers,type,feat);
    if(mdef) return mdef;
     mdef=VerilogDocGen::findDefinition(cd,memName);
      if(mdef)
	    return mdef;  
   }


	 mdef=VerilogDocGen::findMemberDef(cd,memName,MemberList::pubMethods,type,false);
    
	return mdef;

}//findMember


MemberDef* VerilogDocGen::findDefinition(ClassDef *cd, QCString& memName){
 MemberDef *md;
 MemberList *ml=	cd->getMemberList(MemberList::variableMembers);
  if(ml==NULL) return NULL;
    MemberListIterator fmni(*ml);
     
	    for (fmni.toFirst();(md=fmni.current());++fmni)
        {
            if(md->getMemberSpecifiers()==VerilogDocGen::INCLUDE){
          
		
			ClassDef* cdef=getClass(md->name());
			 if(cdef){	 
			    MemberDef* mdd=VerilogDocGen::findMemberDef(cdef,memName,MemberList::variableMembers,-1,false);
               MemberList *ml=	cdef->getMemberList(MemberList::variableMembers);
			   assert(ml);
			   if(mdd) return mdd;
			  MemberListIterator fmni(*ml);
      

			  //assert(false);
			 }
		  }
		}//for
 return NULL;
}//findDefinition


 MemberDef* VerilogDocGen::findMemberDef(ClassDef* cd,QCString& key,MemberList::ListType type,int spec,bool def)
 {
	 static QRegExp regg("[_a-zA-Z]");
	 MemberDef *md=NULL;
 
     MemberList *ml=	cd->getMemberList(type);
     if(ml==NULL) return NULL;
	 
	   MemberListIterator fmni(*ml);
      
	    for (fmni.toFirst();(md=fmni.current());++fmni)
        {
          QCString nn(md->name());
		  int j=nn.find(regg,0);
          if (j>0)
          {
           nn=nn.mid(j,nn.length());
          }
		  if(spec!=-1){ 
		  if((stricmp(key.data(),nn.data())==0) && (md->getMemberSpecifiers()==spec)) 
			  return md;
			  }
          else {
                 if(stricmp(key.data(),nn.data())==0) 
			     return md;
		       }
        } 

 return NULL;

}//findMemberDef

// writes an identifier to the source
void VerilogDocGen::parseIdentVerilog(){
  if(parseCode) 
	                      {
	                      	writePrevVerilogWords(identVerilog);
	                      	writeVerilogFont("vhdllogic",identVerilog.data());
	                      	identVerilog.resize(0);
	                      	printVerilogBuffer(true);
	                      }
              }
 
 // sets the current parsing module (only for inline_sources)             
 void VerilogDocGen::setCurrVerilogClass(QCString& cl){ currVerilogClass = cl;}
   
 //-------------------------------------------------------------------------------------------  
           
 int MyParserConv::parse(MyParserConv* conv){
  myconv=conv;
  assert(myconv);
  return c_parse();
 } 
        
int c_lex(void){
 return myconv->doLex(); 
}


void c_error(const char * err){
   if(err && !parseCode){
   fprintf(stderr,"error occurred at line [%d] : in file [%s]",c_lloc.first_line,getVerilogParsingFile());
  verilogBufferClear();
  }
  // exit(0);
   } 
    
int getVerilogToken(){return c_char;}
 //------------------------------------------------------------------------------------------------  

// writes a digit to the source

void writeDigit()
 {
   if(parseCode) {
     writePrevVerilogWords(identVerilog);
	 writeVerilogFont("vhdllogic",identVerilog.data());
	 identVerilog.resize(0);
	 printVerilogBuffer(true);
	 }
 }// writeDigit

// prints and links the parsed identifiers  

void parseString(){				
					if(parseCode ) { 
					 //   printVerilogStringList();
						 identVerilog=identVerilog.stripWhiteSpace();
				   	  writePrevVerilogWords(identVerilog);
						 bool b=false;
					 
					 if(currVerilogType==VerilogDocGen::DEFPARAM){
				       QCString s(getVerilogString());
                       if(s.contains(".")==0)
                           b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::COMPONENT);
				       else if(s.contains("="))
                           b=generateVerilogMemLink(currVerilogClass,identVerilog,-1);
                       else
				         b=generateVerilogMemLink(currVerilogInst,identVerilog,-1);	       
				     }
					 else if(currVerilogType==VerilogDocGen::COMPONENT){
					    QCString tt(getVerilogString());
					    if(tt.contains('('))
					     b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::PORT);
				        else if(!b)   
				         b=generateVerilogMemLink(currVerilogInst,identVerilog,VerilogDocGen::PORT);
				        if(!b)   
				         b=generateVerilogMemLink(currVerilogClass,identVerilog,-1);    
					   }
				      else if(currVerilogType==VerilogDocGen::NETTYPE){
                       QCString tt(getVerilogString());
                      if(tt.contains("["))
                         b=generateVerilogMemLink(currVerilogClass,identVerilog,-1);
                       else{
                      	 codifyVerilogString(identVerilog.data(),"vhdlcharacter");
				         b=true;
				          }
                      	 }
				      else if(currVerilogType==VerilogDocGen::PORT)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::PORT);
				     else if(currVerilogType==VerilogDocGen::PARAMETER)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::PARAMETER);
				     else if(currVerilogType==VerilogDocGen::REGISTER)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::REGISTER);
				     else if(currVerilogType==VerilogDocGen::INPUT)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::INPUT);				          else if(currVerilogType==VerilogDocGen::INOUT)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::OUTPUT);
				     else if(currVerilogType==VerilogDocGen::OUTPUT)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::INOUT);
				     else if(currVerilogType==VerilogDocGen::ALWAYS)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::ALWAYS);
				     else if(currVerilogType==VerilogDocGen::INTEGER)
                        b=generateVerilogMemLink(currVerilogClass,identVerilog,VerilogDocGen::INTEGER);
					
				     if(!b){
					   b =  generateVerilogMemLink(currVerilogClass,identVerilog,-1); 
					   if(!b && getClass(identVerilog.data()))
                       b=generateVerilogClassOrGlobalLink(identVerilog.data());
					  if(!b){
					   int  col=VerilogDocGen::findKeyWord(identVerilog.data());
					   if(col==1)
					    codifyVerilogString(identVerilog.data(),"vhdldigit");
					   else if(col==2)
					     codifyVerilogString(identVerilog.data(),"comment"); 
					    else
					   codifyVerilogString(identVerilog.data(),"vhdlcharacter");
					   }   
					 }
					   printVerilogBuffer(true);
					  }
				   identVerilog.resize(0);
				 
}// parseString

// inits the parser

void VerilogDocGen::initVerilogParser(){
  currVerilogInst.resize(0);
  currVerilogClass.resize(0);
  prevDocEntryVerilog.reset();

  currentVerilog=0;
  currentFunctionVerilog=0;
}

 //---------------------------------------------------------------------------------------------------  

// adds all gathered [ `include ] to the module

void VerilogDocGen::addIncludes(){
QList<Entry>qll= getVerilogFeatureList();
if(!qll.isEmpty() && currentVerilog==0)
{

  
  QCString file(getVerilogParsingFile());
  QStringList ql=QStringList::split('/',file,false);
  file=(QCString)ql.last();
  int j=file.find(".");
  if(j>0)
   file=file.left(j);

 currentVerilog=VerilogDocGen::makeNewEntry(file.data(),Entry::CLASS_SEC,VerilogDocGen::INCLUDE);
 currentVerilog->protection=Public;
					                       
 mapVerilogFeatures(currentVerilog);
 VerilogDocGen::addSubEntry(current_rootVerilog,currentVerilog);
}
}//cleanUp


// returns the name of module/primitive

QCString VerilogDocGen::getClassTitle(const ClassDef* cdef){
if(cdef->protection()==Public)
  return cdef->className()+" Module";
return cdef->className()+" Primitive";
}// getClassTitle


// do not include the same class twice 

bool findExtendsComponent(QList<BaseInfo> *extend,QCString& compName)
{
 for(uint j=0;j<extend->count();j++){
  BaseInfo *bb=extend->at(j);
  if(bb->name==compName)
   return true;
 }
 return false;
}// findExtendsComponent